/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

GtkWidget* create_mainwindow (void);
GtkWidget* create_aboutwindow (void);
GtkWidget* create_errordialog (void);
GtkWidget* create_terminalwindow (void);
GtkWidget* create_chmemorywindow (void);
GtkWidget* create_chprogwindow (void);
GtkWidget* create_freqlistwindow (void);
GtkWidget* create_newstationwindow (void);
GtkWidget* create_modemselectwindow (void);
GtkWidget* create_configwindow (void);
GtkWidget* create_dtmfwindow (void);
GtkWidget* create_spkscopewindow (void);
GtkWidget* create_spkspectrumwindow (void);
GtkWidget* create_micscopewindow (void);
GtkWidget* create_micspectrumwindow (void);
GtkWidget* create_paramwindow (void);
GtkWidget* create_audiopwrwindow (void);
