/*****************************************************************************/

/*
 *      /home/sailer/src/bayusb/firmware/resetfirmware.h  --  Anchorchips Firmware.
 *
 *      Copyright (C) 1999  Thomas Sailer (sailer@ife.ee.ethz.ch)
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Please note that the GPL allows you to use the driver, NOT the radio.
 *  In order to use the radio, you need a license from the communications
 *  authority of your country.
 *
 */

/*****************************************************************************/
/* automatically generated, do not edit */

#define resetfirmware_DATE "20010821"

/* firmware */
static const unsigned char resetfirmware[] = {
	0x75, 0x81, 0x7f, 0xe4, 0xf5, 0xd0, 0xf5, 0x86, 0x75, 0x8e, 0x31, 0xf5, 0xa8, 0xf5, 0xe8, 0x90, 
	0x7f, 0xae, 0xf0, 0x90, 0x7f, 0xac, 0xf0, 0x90, 0x7f, 0xad, 0xf0, 0x90, 0xe8, 0x90, 0xa3, 0xe5, 
	0x82, 0x45, 0x83, 0x70, 0xf9, 0x90, 0x7f, 0xd6, 0xe4, 0xf0, 0x90, 0x7f, 0xa1, 0x74, 0x01, 0xf0, 
	0x90, 0x7f, 0xaf, 0x74, 0x01, 0xf0, 0xe4, 0x90, 0x7f, 0xe0, 0xf0, 0x90, 0x7f, 0xe1, 0xf0, 0x90, 
	0x7f, 0xdd, 0xf0, 0x90, 0x7f, 0xde, 0xf0, 0x90, 0x7f, 0xdf, 0xf0, 0x90, 0x7f, 0xd7, 0x7f, 0x07, 
	0xef, 0x44, 0x10, 0xf0, 0x44, 0x30, 0xf0, 0xef, 0xf0, 0x44, 0x20, 0xf0, 0xdf, 0xf2, 0x90, 0x7f, 
	0x93, 0x74, 0x00, 0xf0, 0x90, 0x7f, 0x96, 0x74, 0x80, 0xf0, 0x90, 0x7f, 0x9c, 0x74, 0xc2, 0xf0, 
	0x90, 0x7f, 0x94, 0x74, 0x00, 0xf0, 0x90, 0x7f, 0x9d, 0x74, 0x00, 0xf0, 0x90, 0x7f, 0x95, 0x74, 
	0x00, 0xf0, 0x90, 0x7f, 0x98, 0x74, 0x08, 0xf0, 0x90, 0x7f, 0x9e, 0x74, 0x2e, 0xf0, 0x7a, 0x64, 
	0x90, 0xe8, 0x90, 0xa3, 0xe5, 0x82, 0x45, 0x83, 0x70, 0xf9, 0xda, 0xf4, 0x90, 0x7f, 0xd6, 0x74, 
	0x04, 0xf0, 0x90, 0x7f, 0x98, 0x74, 0x20, 0xf0, 0x90, 0x7f, 0x98, 0x74, 0x20, 0xf0, 0x7a, 0x28, 
	0x90, 0xe8, 0x90, 0xa3, 0xe5, 0x82, 0x45, 0x83, 0x70, 0xf9, 0xda, 0xf4, 0x90, 0x7f, 0x98, 0x74, 
	0x28, 0xf0, 0x7a, 0x0a, 0x90, 0xe8, 0x90, 0xa3, 0xe5, 0x82, 0x45, 0x83, 0x70, 0xf9, 0xda, 0xf4, 
	0x80, 0xd6
};
