#define IDD_DIALOG1                     101
#define IDC_SERIAL                      1000
#define IDC_MODE                        1001
#define IDC_BITRATERX                   1002
#define IDC_BITRATETX                   1003
#define IDC_TXDELAY                     1004
#define IDC_FULLDUPLEX                  1005
#define IDC_PTTMUTE                     1006
