/*
 * This file was generated by orbit-idl - DO NOT EDIT!
 */

#include <string.h>
#include "trxctrl.h"

#if ( (TC_IMPL_TC_Trx_frequency_0 == 't') \
&& (TC_IMPL_TC_Trx_frequency_1 == 'r') \
&& (TC_IMPL_TC_Trx_frequency_2 == 'x') \
&& (TC_IMPL_TC_Trx_frequency_3 == 'c') \
&& (TC_IMPL_TC_Trx_frequency_4 == 't') \
&& (TC_IMPL_TC_Trx_frequency_5 == 'r') \
&& (TC_IMPL_TC_Trx_frequency_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_frequency)
#define TC_DEF_TC_Trx_frequency 1
static const CORBA_TypeCode anon_subtypes_array7[] =
   { (CORBA_TypeCode) & TC_CORBA_unsigned_long_long_struct };
const struct CORBA_TypeCode_struct TC_Trx_frequency_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_alias, "frequency", "IDL:Trx/frequency:1.0",
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array7,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_signalstrength_0 == 't') \
&& (TC_IMPL_TC_Trx_signalstrength_1 == 'r') \
&& (TC_IMPL_TC_Trx_signalstrength_2 == 'x') \
&& (TC_IMPL_TC_Trx_signalstrength_3 == 'c') \
&& (TC_IMPL_TC_Trx_signalstrength_4 == 't') \
&& (TC_IMPL_TC_Trx_signalstrength_5 == 'r') \
&& (TC_IMPL_TC_Trx_signalstrength_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_signalstrength)
#define TC_DEF_TC_Trx_signalstrength 1
static const CORBA_TypeCode anon_subtypes_array16[] =
   { (CORBA_TypeCode) & TC_CORBA_float_struct };
const struct CORBA_TypeCode_struct TC_Trx_signalstrength_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_alias, "signalstrength", "IDL:Trx/signalstrength:1.0",
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array16,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxBandDescription_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxBandDescription_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxBandDescription_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxBandDescription_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxBandDescription_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxBandDescription_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxBandDescription_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxBandDescription)
#define TC_DEF_TC_Trx_TrxBandDescription 1
static const char *anon_subnames_array19[] = { "low", "high", "step" };
static const CORBA_TypeCode anon_subtypes_array20[] =
   { (CORBA_TypeCode) & TC_Trx_frequency_struct,
(CORBA_TypeCode) & TC_Trx_frequency_struct, (CORBA_TypeCode) & TC_Trx_frequency_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxBandDescription_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "TrxBandDescription", "IDL:Trx/TrxBandDescription:1.0",
   0, 3,
   (const char **) anon_subnames_array19,
   (CORBA_TypeCode *) anon_subtypes_array20,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_0 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_1 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_2 == 'x') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_3 == 'c') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_4 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_5 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_6 == 'l') \
) && !defined(TC_DEF_TC_CORBA_sequence_Trx_TrxBandDescription)
#define TC_DEF_TC_CORBA_sequence_Trx_TrxBandDescription 1
static const CORBA_TypeCode anon_subtypes_array24[] =
   { (CORBA_TypeCode) & TC_Trx_TrxBandDescription_struct };
const struct CORBA_TypeCode_struct
   TC_CORBA_sequence_Trx_TrxBandDescription_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_sequence, NULL, NULL,
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array24,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_0 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_1 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_2 == 'x') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_3 == 'c') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_4 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_5 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_6 == 'l') \
) && !defined(TC_DEF_TC_CORBA_sequence_Trx_TrxBandDescription)
#define TC_DEF_TC_CORBA_sequence_Trx_TrxBandDescription 1
static const CORBA_TypeCode anon_subtypes_array28[] =
   { (CORBA_TypeCode) & TC_Trx_TrxBandDescription_struct };
const struct CORBA_TypeCode_struct
   TC_CORBA_sequence_Trx_TrxBandDescription_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_sequence, NULL, NULL,
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array28,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxDescription_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxDescription_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxDescription_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxDescription_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxDescription_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxDescription_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxDescription_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxDescription)
#define TC_DEF_TC_Trx_TrxDescription 1
static const char *anon_subnames_array31[] =
   { "manufacturer", "model", "copyright", "instid", "rxbands", "txbands",
"rssimin", "rssimax" };
static const CORBA_TypeCode anon_subtypes_array32[] =
   { (CORBA_TypeCode) & TC_CORBA_string_struct,
(CORBA_TypeCode) & TC_CORBA_string_struct, (CORBA_TypeCode) & TC_CORBA_string_struct,
(CORBA_TypeCode) & TC_CORBA_string_struct, (CORBA_TypeCode) & TC_CORBA_sequence_Trx_TrxBandDescription_struct,
(CORBA_TypeCode) & TC_CORBA_sequence_Trx_TrxBandDescription_struct, (CORBA_TypeCode) & TC_Trx_signalstrength_struct,
(CORBA_TypeCode) & TC_Trx_signalstrength_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxDescription_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "TrxDescription", "IDL:Trx/TrxDescription:1.0",
   0, 8,
   (const char **) anon_subnames_array31,
   (CORBA_TypeCode *) anon_subtypes_array32,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxMode_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxMode_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxMode_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxMode_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxMode_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxMode_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxMode_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxMode)
#define TC_DEF_TC_Trx_TrxMode 1
static const char *anon_subnames_array35[] =
   { "trxunspec", "trxpacket", "trxaudio" };
const struct CORBA_TypeCode_struct TC_Trx_TrxMode_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_enum, "TrxMode", "IDL:Trx/TrxMode:1.0",
   0, 3,
   (const char **) anon_subnames_array35,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxState_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxState_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxState_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxState_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxState_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxState_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxState_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxState)
#define TC_DEF_TC_Trx_TrxState 1
static const char *anon_subnames_array39[] =
   { "mode", "rxfreq", "txfreq", "ptt", "manualptt", "dcd", "rssi" };
static const CORBA_TypeCode anon_subtypes_array40[] =
   { (CORBA_TypeCode) & TC_Trx_TrxMode_struct,
(CORBA_TypeCode) & TC_Trx_frequency_struct, (CORBA_TypeCode) & TC_Trx_frequency_struct,
(CORBA_TypeCode) & TC_CORBA_boolean_struct, (CORBA_TypeCode) & TC_CORBA_boolean_struct,
(CORBA_TypeCode) & TC_CORBA_boolean_struct, (CORBA_TypeCode) & TC_Trx_signalstrength_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxState_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "TrxState", "IDL:Trx/TrxState:1.0",
   0, 7,
   (const char **) anon_subnames_array39,
   (CORBA_TypeCode *) anon_subtypes_array40,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_0 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_1 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_2 == 'x') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_3 == 'c') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_4 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_5 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterMode_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_BaycomUSBAdapterMode)
#define TC_DEF_TC_Trx_BaycomUSBAdapterMode 1
static const char *anon_subnames_array43[] =
   { "fsk", "external", "afsk", "audio" };
const struct CORBA_TypeCode_struct TC_Trx_BaycomUSBAdapterMode_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_enum, "BaycomUSBAdapterMode", "IDL:Trx/BaycomUSBAdapterMode:1.0",
   0, 4,
   (const char **) anon_subnames_array43,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_0 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_1 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_2 == 'x') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_3 == 'c') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_4 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_5 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_BaycomUSBAdapterLoopback)
#define TC_DEF_TC_Trx_BaycomUSBAdapterLoopback 1
static const char *anon_subnames_array47[] =
   { "loopback_off", "loopback_scrambler", "loopback_modem" };
const struct CORBA_TypeCode_struct TC_Trx_BaycomUSBAdapterLoopback_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_enum, "BaycomUSBAdapterLoopback",
      "IDL:Trx/BaycomUSBAdapterLoopback:1.0",
   0, 3,
   (const char **) anon_subnames_array47,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_0 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_1 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_2 == 'x') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_3 == 'c') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_4 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_5 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_BaycomUSBAdapterAudioDev)
#define TC_DEF_TC_Trx_BaycomUSBAdapterAudioDev 1
static const CORBA_TypeCode anon_subtypes_array62[] =
   { (CORBA_TypeCode) & TC_CORBA_char_struct };
static const struct CORBA_TypeCode_struct anonTC_59_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_array, NULL, NULL,
   64, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array62,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
static const CORBA_TypeCode anon_subtypes_array57[] =
   { (CORBA_TypeCode) & anonTC_59_struct };
const struct CORBA_TypeCode_struct TC_Trx_BaycomUSBAdapterAudioDev_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_alias, "BaycomUSBAdapterAudioDev",
      "IDL:Trx/BaycomUSBAdapterAudioDev:1.0",
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array57,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_0 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_1 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_2 == 'x') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_3 == 'c') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_4 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_5 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_BaycomUSBAdapterConfig)
#define TC_DEF_TC_Trx_BaycomUSBAdapterConfig 1
static const char *anon_subnames_array65[] =
   { "mode", "fclk", "bitraterx", "bitratetx", "loopback", "pttmute",
"filtmode", "samplerate", "gain", "audiodevin", "audiodevout", "rfsquelch", "audiosquelch" };
static const CORBA_TypeCode anon_subtypes_array66[] =
   { (CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterMode_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterLoopback_struct,
(CORBA_TypeCode) & TC_CORBA_boolean_struct, (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_CORBA_float_struct,
(CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterAudioDev_struct, (CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterAudioDev_struct,
(CORBA_TypeCode) & TC_Trx_signalstrength_struct, (CORBA_TypeCode) & TC_CORBA_boolean_struct };
const struct CORBA_TypeCode_struct TC_Trx_BaycomUSBAdapterConfig_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "BaycomUSBAdapterConfig",
      "IDL:Trx/BaycomUSBAdapterConfig:1.0",
   0, 13,
   (const char **) anon_subnames_array65,
   (CORBA_TypeCode *) anon_subtypes_array66,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 == 'x') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 == 'c') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 == 'l') \
) && !defined(TC_DEF_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev)
#define TC_DEF_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev 1
static const CORBA_TypeCode anon_subtypes_array70[] =
   { (CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterAudioDev_struct };
const struct CORBA_TypeCode_struct
   TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_sequence, NULL, NULL,
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array70,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 == 'x') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 == 'c') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 == 'l') \
) && !defined(TC_DEF_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev)
#define TC_DEF_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev 1
static const CORBA_TypeCode anon_subtypes_array79[] =
   { (CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterAudioDev_struct };
const struct CORBA_TypeCode_struct
   TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_sequence, NULL, NULL,
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array79,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_0 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_1 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_2 == 'x') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_3 == 'c') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_4 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_5 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_BaycomUSBAdapterAudioDevSequence)
#define TC_DEF_TC_Trx_BaycomUSBAdapterAudioDevSequence 1
static const CORBA_TypeCode anon_subtypes_array83[] =
   { (CORBA_TypeCode) & TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_struct
   };
const struct CORBA_TypeCode_struct
   TC_Trx_BaycomUSBAdapterAudioDevSequence_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_alias, "BaycomUSBAdapterAudioDevSequence",
      "IDL:Trx/BaycomUSBAdapterAudioDevSequence:1.0",
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array83,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_0 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_1 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_2 == 'x') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_3 == 'c') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_4 == 't') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_5 == 'r') \
&& (TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_BaycomUSBAdapterAudioDevs)
#define TC_DEF_TC_Trx_BaycomUSBAdapterAudioDevs 1
static const char *anon_subnames_array86[] =
   { "audiodevsin", "audiodevsout" };
static const CORBA_TypeCode anon_subtypes_array87[] =
   { (CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterAudioDevSequence_struct,
(CORBA_TypeCode) & TC_Trx_BaycomUSBAdapterAudioDevSequence_struct };
const struct CORBA_TypeCode_struct TC_Trx_BaycomUSBAdapterAudioDevs_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "BaycomUSBAdapterAudioDevs",
      "IDL:Trx/BaycomUSBAdapterAudioDevs:1.0",
   0, 2,
   (const char **) anon_subnames_array86,
   (CORBA_TypeCode *) anon_subtypes_array87,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_ChannelAccessConfig_0 == 't') \
&& (TC_IMPL_TC_Trx_ChannelAccessConfig_1 == 'r') \
&& (TC_IMPL_TC_Trx_ChannelAccessConfig_2 == 'x') \
&& (TC_IMPL_TC_Trx_ChannelAccessConfig_3 == 'c') \
&& (TC_IMPL_TC_Trx_ChannelAccessConfig_4 == 't') \
&& (TC_IMPL_TC_Trx_ChannelAccessConfig_5 == 'r') \
&& (TC_IMPL_TC_Trx_ChannelAccessConfig_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_ChannelAccessConfig)
#define TC_DEF_TC_Trx_ChannelAccessConfig 1
static const char *anon_subnames_array90[] =
   { "txdelay", "slottime", "ppersistence", "txtail", "fullduplex" };
static const CORBA_TypeCode anon_subtypes_array91[] =
   { (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_CORBA_boolean_struct };
const struct CORBA_TypeCode_struct TC_Trx_ChannelAccessConfig_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "ChannelAccessConfig", "IDL:Trx/ChannelAccessConfig:1.0",
   0, 5,
   (const char **) anon_subnames_array90,
   (CORBA_TypeCode *) anon_subtypes_array91,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_IpAddress_0 == 't') \
&& (TC_IMPL_TC_Trx_IpAddress_1 == 'r') \
&& (TC_IMPL_TC_Trx_IpAddress_2 == 'x') \
&& (TC_IMPL_TC_Trx_IpAddress_3 == 'c') \
&& (TC_IMPL_TC_Trx_IpAddress_4 == 't') \
&& (TC_IMPL_TC_Trx_IpAddress_5 == 'r') \
&& (TC_IMPL_TC_Trx_IpAddress_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_IpAddress)
#define TC_DEF_TC_Trx_IpAddress 1
static const CORBA_TypeCode anon_subtypes_array105[] =
   { (CORBA_TypeCode) & TC_CORBA_octet_struct };
static const struct CORBA_TypeCode_struct anonTC_102_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_array, NULL, NULL,
   4, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array105,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
static const CORBA_TypeCode anon_subtypes_array100[] =
   { (CORBA_TypeCode) & anonTC_102_struct };
const struct CORBA_TypeCode_struct TC_Trx_IpAddress_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_alias, "IpAddress", "IDL:Trx/IpAddress:1.0",
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array100,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_InterfaceConfig_0 == 't') \
&& (TC_IMPL_TC_Trx_InterfaceConfig_1 == 'r') \
&& (TC_IMPL_TC_Trx_InterfaceConfig_2 == 'x') \
&& (TC_IMPL_TC_Trx_InterfaceConfig_3 == 'c') \
&& (TC_IMPL_TC_Trx_InterfaceConfig_4 == 't') \
&& (TC_IMPL_TC_Trx_InterfaceConfig_5 == 'r') \
&& (TC_IMPL_TC_Trx_InterfaceConfig_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_InterfaceConfig)
#define TC_DEF_TC_Trx_InterfaceConfig 1
static const char *anon_subnames_array108[] =
   { "ifname", "hwaddr", "ipaddr", "netmask", "broadcast", "gwhwaddr",
"gwipaddr", "gwnetmask", "gwipmode" };
static const CORBA_TypeCode anon_subtypes_array109[] =
   { (CORBA_TypeCode) & TC_CORBA_char_struct,
(CORBA_TypeCode) & TC_CORBA_char_struct, (CORBA_TypeCode) & TC_Trx_IpAddress_struct,
(CORBA_TypeCode) & TC_Trx_IpAddress_struct, (CORBA_TypeCode) & TC_Trx_IpAddress_struct,
(CORBA_TypeCode) & TC_CORBA_char_struct, (CORBA_TypeCode) & TC_Trx_IpAddress_struct,
(CORBA_TypeCode) & TC_Trx_IpAddress_struct, (CORBA_TypeCode) & TC_CORBA_char_struct };
const struct CORBA_TypeCode_struct TC_Trx_InterfaceConfig_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "InterfaceConfig", "IDL:Trx/InterfaceConfig:1.0",
   0, 9,
   (const char **) anon_subnames_array108,
   (CORBA_TypeCode *) anon_subtypes_array109,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_ModemDisconnectRxC_0 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectRxC_1 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectRxC_2 == 'x') \
&& (TC_IMPL_TC_Trx_ModemDisconnectRxC_3 == 'c') \
&& (TC_IMPL_TC_Trx_ModemDisconnectRxC_4 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectRxC_5 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectRxC_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_ModemDisconnectRxC)
#define TC_DEF_TC_Trx_ModemDisconnectRxC 1
static const char *anon_subnames_array112[] =
   { "mdisc_rxc_normal", "mdisc_rxc_rxclock" };
const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectRxC_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_enum, "ModemDisconnectRxC", "IDL:Trx/ModemDisconnectRxC:1.0",
   0, 2,
   (const char **) anon_subnames_array112,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_ModemDisconnectTxC_0 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxC_1 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxC_2 == 'x') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxC_3 == 'c') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxC_4 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxC_5 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxC_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_ModemDisconnectTxC)
#define TC_DEF_TC_Trx_ModemDisconnectTxC 1
static const char *anon_subnames_array116[] =
   { "mdisc_txc_normal", "mdisc_txc_txclock" };
const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectTxC_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_enum, "ModemDisconnectTxC", "IDL:Trx/ModemDisconnectTxC:1.0",
   0, 2,
   (const char **) anon_subnames_array116,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_ModemDisconnectTxD_0 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxD_1 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxD_2 == 'x') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxD_3 == 'c') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxD_4 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxD_5 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectTxD_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_ModemDisconnectTxD)
#define TC_DEF_TC_Trx_ModemDisconnectTxD 1
static const char *anon_subnames_array120[] =
   { "mdisc_txd_normal", "mdisc_txd_txdata" };
const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectTxD_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_enum, "ModemDisconnectTxD", "IDL:Trx/ModemDisconnectTxD:1.0",
   0, 2,
   (const char **) anon_subnames_array120,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_ModemDisconnectConfig_0 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectConfig_1 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectConfig_2 == 'x') \
&& (TC_IMPL_TC_Trx_ModemDisconnectConfig_3 == 'c') \
&& (TC_IMPL_TC_Trx_ModemDisconnectConfig_4 == 't') \
&& (TC_IMPL_TC_Trx_ModemDisconnectConfig_5 == 'r') \
&& (TC_IMPL_TC_Trx_ModemDisconnectConfig_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_ModemDisconnectConfig)
#define TC_DEF_TC_Trx_ModemDisconnectConfig 1
static const char *anon_subnames_array124[] =
   { "output", "direction", "rxc", "txc", "txd" };
static const CORBA_TypeCode anon_subtypes_array125[] =
   { (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_Trx_ModemDisconnectRxC_struct,
(CORBA_TypeCode) & TC_Trx_ModemDisconnectTxC_struct, (CORBA_TypeCode) & TC_Trx_ModemDisconnectTxD_struct };
const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectConfig_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "ModemDisconnectConfig",
      "IDL:Trx/ModemDisconnectConfig:1.0",
   0, 5,
   (const char **) anon_subnames_array124,
   (CORBA_TypeCode *) anon_subtypes_array125,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxAudioState_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxAudioState_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxAudioState_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxAudioState_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxAudioState_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxAudioState_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxAudioState_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxAudioState)
#define TC_DEF_TC_Trx_TrxAudioState 1
static const char *anon_subnames_array128[] =
   { "samplerate", "micbufsz", "micbufptr", "spkbufsz", "spkbufptr",
"micsigpwr", "spksigpwr", "spknoisepwr" };
static const CORBA_TypeCode anon_subtypes_array129[] =
   { (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_unsigned_long_struct, (CORBA_TypeCode) & TC_CORBA_unsigned_long_struct,
(CORBA_TypeCode) & TC_CORBA_float_struct, (CORBA_TypeCode) & TC_CORBA_float_struct,
(CORBA_TypeCode) & TC_CORBA_float_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxAudioState_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "TrxAudioState", "IDL:Trx/TrxAudioState:1.0",
   0, 8,
   (const char **) anon_subnames_array128,
   (CORBA_TypeCode *) anon_subtypes_array129,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_CORBA_sequence_CORBA_octet_0 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_1 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_2 == 'x') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_3 == 'c') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_4 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_5 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_6 == 'l') \
) && !defined(TC_DEF_TC_CORBA_sequence_CORBA_octet)
#define TC_DEF_TC_CORBA_sequence_CORBA_octet 1
static const CORBA_TypeCode anon_subtypes_array133[] =
   { (CORBA_TypeCode) & TC_CORBA_octet_struct };
const struct CORBA_TypeCode_struct TC_CORBA_sequence_CORBA_octet_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_sequence, NULL, NULL,
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array133,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_CORBA_sequence_CORBA_octet_0 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_1 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_2 == 'x') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_3 == 'c') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_4 == 't') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_5 == 'r') \
&& (TC_IMPL_TC_CORBA_sequence_CORBA_octet_6 == 'l') \
) && !defined(TC_DEF_TC_CORBA_sequence_CORBA_octet)
#define TC_DEF_TC_CORBA_sequence_CORBA_octet 1
static const CORBA_TypeCode anon_subtypes_array142[] =
   { (CORBA_TypeCode) & TC_CORBA_octet_struct };
const struct CORBA_TypeCode_struct TC_CORBA_sequence_CORBA_octet_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_sequence, NULL, NULL,
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array142,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxAudioSamples_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxAudioSamples_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxAudioSamples_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxAudioSamples_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxAudioSamples_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxAudioSamples_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxAudioSamples_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxAudioSamples)
#define TC_DEF_TC_Trx_TrxAudioSamples 1
static const CORBA_TypeCode anon_subtypes_array146[] =
   { (CORBA_TypeCode) & TC_CORBA_sequence_CORBA_octet_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxAudioSamples_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_alias, "TrxAudioSamples", "IDL:Trx/TrxAudioSamples:1.0",
   0, 1,
   NULL,
   (CORBA_TypeCode *) anon_subtypes_array146,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxAudioParams_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxAudioParams_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxAudioParams_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxAudioParams_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxAudioParams_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxAudioParams_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxAudioParams_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxAudioParams)
#define TC_DEF_TC_Trx_TrxAudioParams 1
static const char *anon_subnames_array149[] = { "rfsquelch", "audiosquelch" };
static const CORBA_TypeCode anon_subtypes_array150[] =
   { (CORBA_TypeCode) & TC_Trx_signalstrength_struct,
(CORBA_TypeCode) & TC_CORBA_boolean_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxAudioParams_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_struct, "TrxAudioParams", "IDL:Trx/TrxAudioParams:1.0",
   0, 2,
   (const char **) anon_subnames_array149,
   (CORBA_TypeCode *) anon_subtypes_array150,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_TrxError_0 == 't') \
&& (TC_IMPL_TC_Trx_TrxError_1 == 'r') \
&& (TC_IMPL_TC_Trx_TrxError_2 == 'x') \
&& (TC_IMPL_TC_Trx_TrxError_3 == 'c') \
&& (TC_IMPL_TC_Trx_TrxError_4 == 't') \
&& (TC_IMPL_TC_Trx_TrxError_5 == 'r') \
&& (TC_IMPL_TC_Trx_TrxError_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_TrxError)
#define TC_DEF_TC_Trx_TrxError 1
static const char *anon_subnames_array153[] = { "reason" };
static const CORBA_TypeCode anon_subtypes_array154[] =
   { (CORBA_TypeCode) & TC_CORBA_string_struct };
const struct CORBA_TypeCode_struct TC_Trx_TrxError_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_except, "TrxError", "IDL:Trx/TrxError:1.0",
   0, 1,
   (const char **) anon_subnames_array153,
   (CORBA_TypeCode *) anon_subtypes_array154,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
#if ( (TC_IMPL_TC_Trx_Control_0 == 't') \
&& (TC_IMPL_TC_Trx_Control_1 == 'r') \
&& (TC_IMPL_TC_Trx_Control_2 == 'x') \
&& (TC_IMPL_TC_Trx_Control_3 == 'c') \
&& (TC_IMPL_TC_Trx_Control_4 == 't') \
&& (TC_IMPL_TC_Trx_Control_5 == 'r') \
&& (TC_IMPL_TC_Trx_Control_6 == 'l') \
) && !defined(TC_DEF_TC_Trx_Control)
#define TC_DEF_TC_Trx_Control 1
const struct CORBA_TypeCode_struct TC_Trx_Control_struct = {
   {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
    ORBIT_PSEUDO_TYPECODE},
   CORBA_tk_objref, "Control", "IDL:Trx/Control:1.0",
   0, 0,
   NULL,
   NULL,
   NULL,
   CORBA_OBJECT_NIL, 0, -1, 0, 0
};
#endif
gpointer
Trx_TrxBandDescription__free(gpointer mem, gpointer dat,
			     CORBA_boolean free_strings)
{
   Trx_TrxBandDescription *var = mem;

   return (gpointer) (var + 1);
}

Trx_TrxBandDescription *
Trx_TrxBandDescription__alloc(void)
{
   Trx_TrxBandDescription *retval;

   retval =
      ORBit_alloc(sizeof(Trx_TrxBandDescription),
		  (ORBit_free_childvals) Trx_TrxBandDescription__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

#if ( (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_0 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_1 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_2 == 'x') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_3 == 'c') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_4 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_5 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_6 == 'l') \
) && !defined(ORBIT_DEF_CORBA_sequence_Trx_TrxBandDescription)
#define ORBIT_DEF_CORBA_sequence_Trx_TrxBandDescription 1

gpointer
CORBA_sequence_Trx_TrxBandDescription__free(gpointer mem, gpointer dat,
					    CORBA_boolean free_strings)
{
   CORBA_sequence_Trx_TrxBandDescription *val = mem;

   if (val->_release)
      ORBit_free(val->_buffer, free_strings);
   return (gpointer) (val + 1);
}

CORBA_sequence_Trx_TrxBandDescription *
CORBA_sequence_Trx_TrxBandDescription__alloc(void)
{
   CORBA_sequence_Trx_TrxBandDescription *retval;

   retval =
      ORBit_alloc(sizeof(CORBA_sequence_Trx_TrxBandDescription),
		  (ORBit_free_childvals)
		  CORBA_sequence_Trx_TrxBandDescription__free,
		  GUINT_TO_POINTER(1));
   retval->_maximum = 0;
   retval->_length = 0;
   retval->_buffer = NULL;
   retval->_release = CORBA_FALSE;
   return retval;
}

Trx_TrxBandDescription *
CORBA_sequence_Trx_TrxBandDescription_allocbuf(CORBA_unsigned_long len)
{
   Trx_TrxBandDescription *retval =
      ORBit_alloc(sizeof(Trx_TrxBandDescription) * len,
		  (ORBit_free_childvals) NULL, GUINT_TO_POINTER(len));
   return retval;
}
#endif

#if ( (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_0 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_1 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_2 == 'x') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_3 == 'c') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_4 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_5 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_6 == 'l') \
) && !defined(ORBIT_DEF_CORBA_sequence_Trx_TrxBandDescription)
#define ORBIT_DEF_CORBA_sequence_Trx_TrxBandDescription 1

gpointer
CORBA_sequence_Trx_TrxBandDescription__free(gpointer mem, gpointer dat,
					    CORBA_boolean free_strings)
{
   CORBA_sequence_Trx_TrxBandDescription *val = mem;

   if (val->_release)
      ORBit_free(val->_buffer, free_strings);
   return (gpointer) (val + 1);
}

CORBA_sequence_Trx_TrxBandDescription *
CORBA_sequence_Trx_TrxBandDescription__alloc(void)
{
   CORBA_sequence_Trx_TrxBandDescription *retval;

   retval =
      ORBit_alloc(sizeof(CORBA_sequence_Trx_TrxBandDescription),
		  (ORBit_free_childvals)
		  CORBA_sequence_Trx_TrxBandDescription__free,
		  GUINT_TO_POINTER(1));
   retval->_maximum = 0;
   retval->_length = 0;
   retval->_buffer = NULL;
   retval->_release = CORBA_FALSE;
   return retval;
}

Trx_TrxBandDescription *
CORBA_sequence_Trx_TrxBandDescription_allocbuf(CORBA_unsigned_long len)
{
   Trx_TrxBandDescription *retval =
      ORBit_alloc(sizeof(Trx_TrxBandDescription) * len,
		  (ORBit_free_childvals) NULL, GUINT_TO_POINTER(len));
   return retval;
}
#endif

gpointer
Trx_TrxDescription__free(gpointer mem, gpointer dat,
			 CORBA_boolean free_strings)
{
   Trx_TrxDescription *var = mem;

   if (free_strings) {
      CORBA_string__free(&(var->manufacturer), NULL, free_strings);
   }
   if (free_strings) {
      CORBA_string__free(&(var->model), NULL, free_strings);
   }
   if (free_strings) {
      CORBA_string__free(&(var->copyright), NULL, free_strings);
   }
   if (free_strings) {
      CORBA_string__free(&(var->instid), NULL, free_strings);
   }
   {
      CORBA_sequence_Trx_TrxBandDescription__free(&(var->rxbands), NULL,
						  free_strings);
   }
   {
      CORBA_sequence_Trx_TrxBandDescription__free(&(var->txbands), NULL,
						  free_strings);
   }
   return (gpointer) (var + 1);
}

Trx_TrxDescription *
Trx_TrxDescription__alloc(void)
{
   Trx_TrxDescription *retval;

   retval =
      ORBit_alloc(sizeof(Trx_TrxDescription),
		  (ORBit_free_childvals) Trx_TrxDescription__free,
		  GUINT_TO_POINTER(1));
   memset(&(retval->manufacturer), '\0', sizeof(retval->manufacturer));
   memset(&(retval->model), '\0', sizeof(retval->model));
   memset(&(retval->copyright), '\0', sizeof(retval->copyright));
   memset(&(retval->instid), '\0', sizeof(retval->instid));
   memset(&(retval->rxbands), '\0', sizeof(retval->rxbands));
   memset(&(retval->txbands), '\0', sizeof(retval->txbands));
   return retval;
}

gpointer
Trx_TrxState__free(gpointer mem, gpointer dat, CORBA_boolean free_strings)
{
   Trx_TrxState *var = mem;

   return (gpointer) (var + 1);
}

Trx_TrxState *
Trx_TrxState__alloc(void)
{
   Trx_TrxState *retval;

   retval =
      ORBit_alloc(sizeof(Trx_TrxState),
		  (ORBit_free_childvals) Trx_TrxState__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

gpointer
Trx_BaycomUSBAdapterAudioDev__free(gpointer mem, gpointer dat,
				   CORBA_boolean free_strings)
{
   gpointer retval = ((guchar *) mem) + sizeof(Trx_BaycomUSBAdapterAudioDev);

   return retval;
}

Trx_BaycomUSBAdapterAudioDev_slice *
Trx_BaycomUSBAdapterAudioDev__alloc(void)
{
   Trx_BaycomUSBAdapterAudioDev_slice *retval;

   retval =
      ORBit_alloc(sizeof(Trx_BaycomUSBAdapterAudioDev),
		  (ORBit_free_childvals) NULL, NULL);
   return retval;
}

gpointer
Trx_BaycomUSBAdapterConfig__free(gpointer mem, gpointer dat,
				 CORBA_boolean free_strings)
{
   Trx_BaycomUSBAdapterConfig *var = mem;

   return (gpointer) (var + 1);
}

Trx_BaycomUSBAdapterConfig *
Trx_BaycomUSBAdapterConfig__alloc(void)
{
   Trx_BaycomUSBAdapterConfig *retval;

   retval =
      ORBit_alloc(sizeof(Trx_BaycomUSBAdapterConfig),
		  (ORBit_free_childvals) Trx_BaycomUSBAdapterConfig__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

#if ( (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 == 'x') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 == 'c') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 == 'l') \
) && !defined(ORBIT_DEF_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev)
#define ORBIT_DEF_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev 1

gpointer
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free(gpointer mem, gpointer dat,
						  CORBA_boolean free_strings)
{
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *val = mem;

   if (val->_release)
      ORBit_free(val->_buffer, free_strings);
   return (gpointer) (val + 1);
}

CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__alloc(void)
{
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *retval;

   retval =
      ORBit_alloc(sizeof(CORBA_sequence_Trx_BaycomUSBAdapterAudioDev),
		  (ORBit_free_childvals)
		  CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free,
		  GUINT_TO_POINTER(1));
   retval->_maximum = 0;
   retval->_length = 0;
   retval->_buffer = NULL;
   retval->_release = CORBA_FALSE;
   return retval;
}

Trx_BaycomUSBAdapterAudioDev *
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_allocbuf(CORBA_unsigned_long len)
{
   Trx_BaycomUSBAdapterAudioDev *retval =
      ORBit_alloc(sizeof(Trx_BaycomUSBAdapterAudioDev) * len,
		  (ORBit_free_childvals) NULL, GUINT_TO_POINTER(len));
   return retval;
}
#endif

gpointer
Trx_BaycomUSBAdapterAudioDevSequence__free(gpointer mem, gpointer dat,
					   CORBA_boolean free_strings)
{
   return CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free(mem, dat,
							    free_strings);
}

Trx_BaycomUSBAdapterAudioDevSequence *
Trx_BaycomUSBAdapterAudioDevSequence__alloc(void)
{
   return CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__alloc();
}

#if ( (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 == 'x') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 == 'c') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 == 'l') \
) && !defined(ORBIT_DEF_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev)
#define ORBIT_DEF_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev 1

gpointer
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free(gpointer mem, gpointer dat,
						  CORBA_boolean free_strings)
{
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *val = mem;

   if (val->_release)
      ORBit_free(val->_buffer, free_strings);
   return (gpointer) (val + 1);
}

CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__alloc(void)
{
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *retval;

   retval =
      ORBit_alloc(sizeof(CORBA_sequence_Trx_BaycomUSBAdapterAudioDev),
		  (ORBit_free_childvals)
		  CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free,
		  GUINT_TO_POINTER(1));
   retval->_maximum = 0;
   retval->_length = 0;
   retval->_buffer = NULL;
   retval->_release = CORBA_FALSE;
   return retval;
}

Trx_BaycomUSBAdapterAudioDev *
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_allocbuf(CORBA_unsigned_long len)
{
   Trx_BaycomUSBAdapterAudioDev *retval =
      ORBit_alloc(sizeof(Trx_BaycomUSBAdapterAudioDev) * len,
		  (ORBit_free_childvals) NULL, GUINT_TO_POINTER(len));
   return retval;
}
#endif

#if ( (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 == 'x') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 == 'c') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 == 'l') \
) && !defined(ORBIT_DEF_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev)
#define ORBIT_DEF_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev 1

gpointer
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free(gpointer mem, gpointer dat,
						  CORBA_boolean free_strings)
{
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *val = mem;

   if (val->_release)
      ORBit_free(val->_buffer, free_strings);
   return (gpointer) (val + 1);
}

CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__alloc(void)
{
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev *retval;

   retval =
      ORBit_alloc(sizeof(CORBA_sequence_Trx_BaycomUSBAdapterAudioDev),
		  (ORBit_free_childvals)
		  CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free,
		  GUINT_TO_POINTER(1));
   retval->_maximum = 0;
   retval->_length = 0;
   retval->_buffer = NULL;
   retval->_release = CORBA_FALSE;
   return retval;
}

Trx_BaycomUSBAdapterAudioDev *
CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_allocbuf(CORBA_unsigned_long len)
{
   Trx_BaycomUSBAdapterAudioDev *retval =
      ORBit_alloc(sizeof(Trx_BaycomUSBAdapterAudioDev) * len,
		  (ORBit_free_childvals) NULL, GUINT_TO_POINTER(len));
   return retval;
}
#endif

gpointer
Trx_BaycomUSBAdapterAudioDevs__free(gpointer mem, gpointer dat,
				    CORBA_boolean free_strings)
{
   Trx_BaycomUSBAdapterAudioDevs *var = mem;

   {
      Trx_BaycomUSBAdapterAudioDevSequence__free(&(var->audiodevsin), NULL,
						 free_strings);
   }
   {
      Trx_BaycomUSBAdapterAudioDevSequence__free(&(var->audiodevsout), NULL,
						 free_strings);
   }
   return (gpointer) (var + 1);
}

Trx_BaycomUSBAdapterAudioDevs *
Trx_BaycomUSBAdapterAudioDevs__alloc(void)
{
   Trx_BaycomUSBAdapterAudioDevs *retval;

   retval =
      ORBit_alloc(sizeof(Trx_BaycomUSBAdapterAudioDevs),
		  (ORBit_free_childvals) Trx_BaycomUSBAdapterAudioDevs__free,
		  GUINT_TO_POINTER(1));
   memset(&(retval->audiodevsin), '\0', sizeof(retval->audiodevsin));
   memset(&(retval->audiodevsout), '\0', sizeof(retval->audiodevsout));
   return retval;
}

gpointer
Trx_ChannelAccessConfig__free(gpointer mem, gpointer dat,
			      CORBA_boolean free_strings)
{
   Trx_ChannelAccessConfig *var = mem;

   return (gpointer) (var + 1);
}

Trx_ChannelAccessConfig *
Trx_ChannelAccessConfig__alloc(void)
{
   Trx_ChannelAccessConfig *retval;

   retval =
      ORBit_alloc(sizeof(Trx_ChannelAccessConfig),
		  (ORBit_free_childvals) Trx_ChannelAccessConfig__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

gpointer
Trx_IpAddress__free(gpointer mem, gpointer dat, CORBA_boolean free_strings)
{
   gpointer retval = ((guchar *) mem) + sizeof(Trx_IpAddress);

   return retval;
}

Trx_IpAddress_slice *
Trx_IpAddress__alloc(void)
{
   Trx_IpAddress_slice *retval;

   retval =
      ORBit_alloc(sizeof(Trx_IpAddress), (ORBit_free_childvals) NULL, NULL);
   return retval;
}

gpointer
Trx_InterfaceConfig__free(gpointer mem, gpointer dat,
			  CORBA_boolean free_strings)
{
   Trx_InterfaceConfig *var = mem;

   return (gpointer) (var + 1);
}

Trx_InterfaceConfig *
Trx_InterfaceConfig__alloc(void)
{
   Trx_InterfaceConfig *retval;

   retval =
      ORBit_alloc(sizeof(Trx_InterfaceConfig),
		  (ORBit_free_childvals) Trx_InterfaceConfig__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

gpointer
Trx_ModemDisconnectConfig__free(gpointer mem, gpointer dat,
				CORBA_boolean free_strings)
{
   Trx_ModemDisconnectConfig *var = mem;

   return (gpointer) (var + 1);
}

Trx_ModemDisconnectConfig *
Trx_ModemDisconnectConfig__alloc(void)
{
   Trx_ModemDisconnectConfig *retval;

   retval =
      ORBit_alloc(sizeof(Trx_ModemDisconnectConfig),
		  (ORBit_free_childvals) Trx_ModemDisconnectConfig__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

gpointer
Trx_TrxAudioState__free(gpointer mem, gpointer dat,
			CORBA_boolean free_strings)
{
   Trx_TrxAudioState *var = mem;

   return (gpointer) (var + 1);
}

Trx_TrxAudioState *
Trx_TrxAudioState__alloc(void)
{
   Trx_TrxAudioState *retval;

   retval =
      ORBit_alloc(sizeof(Trx_TrxAudioState),
		  (ORBit_free_childvals) Trx_TrxAudioState__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

#if ( (ORBIT_IMPL_CORBA_sequence_CORBA_octet_0 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_CORBA_octet_1 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_CORBA_octet_2 == 'x') \
&& (ORBIT_IMPL_CORBA_sequence_CORBA_octet_3 == 'c') \
&& (ORBIT_IMPL_CORBA_sequence_CORBA_octet_4 == 't') \
&& (ORBIT_IMPL_CORBA_sequence_CORBA_octet_5 == 'r') \
&& (ORBIT_IMPL_CORBA_sequence_CORBA_octet_6 == 'l') \
) && !defined(ORBIT_DEF_CORBA_sequence_CORBA_octet)
#define ORBIT_DEF_CORBA_sequence_CORBA_octet 1

gpointer
CORBA_sequence_CORBA_octet__free(gpointer mem, gpointer dat,
				 CORBA_boolean free_strings)
{
   CORBA_sequence_CORBA_octet *val = mem;

   if (val->_release)
      ORBit_free(val->_buffer, free_strings);
   return (gpointer) (val + 1);
}

CORBA_sequence_CORBA_octet *
CORBA_sequence_CORBA_octet__alloc(void)
{
   CORBA_sequence_CORBA_octet *retval;

   retval =
      ORBit_alloc(sizeof(CORBA_sequence_CORBA_octet),
		  (ORBit_free_childvals) CORBA_sequence_CORBA_octet__free,
		  GUINT_TO_POINTER(1));
   retval->_maximum = 0;
   retval->_length = 0;
   retval->_buffer = NULL;
   retval->_release = CORBA_FALSE;
   return retval;
}

CORBA_octet *
CORBA_sequence_CORBA_octet_allocbuf(CORBA_unsigned_long len)
{
   CORBA_octet *retval =
      ORBit_alloc(sizeof(CORBA_octet) * len, (ORBit_free_childvals) NULL,
		  GUINT_TO_POINTER(len));
   return retval;
}
#endif

gpointer
Trx_TrxAudioSamples__free(gpointer mem, gpointer dat,
			  CORBA_boolean free_strings)
{
   return CORBA_sequence_CORBA_octet__free(mem, dat, free_strings);
}

Trx_TrxAudioSamples *
Trx_TrxAudioSamples__alloc(void)
{
   return CORBA_sequence_CORBA_octet__alloc();
}

gpointer
Trx_TrxAudioParams__free(gpointer mem, gpointer dat,
			 CORBA_boolean free_strings)
{
   Trx_TrxAudioParams *var = mem;

   return (gpointer) (var + 1);
}

Trx_TrxAudioParams *
Trx_TrxAudioParams__alloc(void)
{
   Trx_TrxAudioParams *retval;

   retval =
      ORBit_alloc(sizeof(Trx_TrxAudioParams),
		  (ORBit_free_childvals) Trx_TrxAudioParams__free,
		  GUINT_TO_POINTER(1));
   return retval;
}

gpointer
Trx_TrxError__free(gpointer mem, gpointer dat, CORBA_boolean free_strings)
{
   Trx_TrxError *var = mem;

   if (free_strings) {
      CORBA_string__free(&(var->reason), NULL, free_strings);
   }
   return (gpointer) (var + 1);
}

Trx_TrxError *
Trx_TrxError__alloc(void)
{
   Trx_TrxError *retval;

   retval =
      ORBit_alloc(sizeof(Trx_TrxError),
		  (ORBit_free_childvals) Trx_TrxError__free,
		  GUINT_TO_POINTER(1));
   memset(&(retval->reason), '\0', sizeof(retval->reason));
   return retval;
}

CORBA_unsigned_long Trx_Control__classid = 0;
