/* Automatically generated by po2tbl.sed from baycomusb.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"Baycom USB Transceiver Controller", 2},
  {"File", 3},
  {"Frequency List...", 4},
  {"Modem Configuration...", 5},
  {"Select Modem...", 6},
  {"Settings...", 7},
  {"Quit", 8},
  {"Audio", 9},
  {"Speaker Scope", 10},
  {"Speaker Spectrum", 11},
  {"Microphone Scope", 12},
  {"Microphone Spectrum", 13},
  {"DTMF", 14},
  {"Signal Levels", 15},
  {"Expert", 16},
  {"Terminal", 17},
  {"Open Terminal Window", 18},
  {"Channel Window", 19},
  {"Program Channel Memory", 20},
  {"Help", 21},
  {"About", 22},
  {"QRG", 23},
  {"TX", 24},
  {"RX", 25},
  {"MHz", 26},
  {"Station Name", 27},
  {"Status", 28},
  {"PTT", 29},
  {"1750", 30},
  {"DCD", 31},
  {"S", 32},
  {"%v", 33},
  {"S-Meter History", 34},
  {"About Baycom USB Transceiver Controller", 35},
  {"Copyright 2000-2001", 36},
  {"Thomas Sailer, HB9JNX", 37},
  {"Ok", 38},
  {"Error", 39},
  {"OK", 40},
  {"T7F Terminal", 41},
  {"T7F Channel Memory", 42},
  {"Channel", 43},
  {"0", 44},
  {"1", 45},
  {"2", 46},
  {"3", 47},
  {"4", 48},
  {"5", 49},
  {"6", 50},
  {"7", 51},
  {"8", 52},
  {"9", 53},
  {"25kHz Raster", 54},
  {"12.5kHz Raster", 55},
  {"-", 56},
  {"Program Channel", 57},
  {"RX Frequency", 58},
  {"TX Frequency", 59},
  {"Cancel", 60},
  {"Frequency List", 61},
  {"Mode", 62},
  {"Set", 63},
  {"New", 64},
  {"Update", 65},
  {"Delete", 66},
  {"New Station", 67},
  {"Bitrate RX", 68},
  {"Bitrate TX", 69},
  {"fsk", 70},
  {"afsk", 71},
  {"audio", 72},
  {"external", 73},
  {"Bit/s", 74},
  {"Modem/Transceiver Selection", 75},
  {"Manufacturer", 76},
  {"Model", 77},
  {"Serial Number", 78},
  {"Baycom USB Adapter Configuration", 79},
  {"FPGA clock", 80},
  {"Hz", 81},
  {"used only in FSK and EXTERNAL mode", 82},
  {"Bits/s", 83},
  {"Loopback", 84},
  {"PTT Mute", 85},
  {"Filter Mode", 86},
  {"Adapter", 87},
  {"Sample Rate", 88},
  {"Gain", 89},
  {"Samples/s", 90},
  {"used only in AUDIO mode", 91},
  {"Speaker Audio Device", 92},
  {"Microphone Audio Device", 93},
  {"/dev/dsp", 94},
  {"RF Squelch Level", 95},
  {"Audio Squelch", 96},
  {"Full Duplex", 97},
  {"P-Persistence", 98},
  {"SlotTime", 99},
  {"TxTail", 100},
  {"TxDelay", 101},
  {"ms", 102},
  {"Channel Access", 103},
  {"bcu0", 104},
  {"bcu1", 105},
  {"bcu2", 106},
  {"bcu3", 107},
  {"10.0.0.1", 108},
  {"255.255.255.0", 109},
  {"10.0.0.255", 110},
  {"Interface Name", 111},
  {"Callsign", 112},
  {"IP Address", 113},
  {"Netmask", 114},
  {"IP Broadcast Address", 115},
  {"Gateway AX.25 Path", 116},
  {"Gateway IP Address", 117},
  {"Gateway Genmask", 118},
  {"Gateway IP Mode", 119},
  {"Default", 120},
  {"Virtual Circuit", 121},
  {"Datagram", 122},
  {"Compressed", 123},
  {"Interface", 124},
  {"Pin 5", 125},
  {"Pin 7", 126},
  {"Pin 9", 127},
  {"Pin 11", 128},
  {"Pin 13", 129},
  {"Pin 15", 130},
  {"Pin 17", 131},
  {"Pin 19", 132},
  {"Input", 133},
  {"RXC", 134},
  {"TXC", 135},
  {"RXD", 136},
  {"TXD", 137},
  {"/RTS", 138},
  {"/CTS", 139},
  {"/DCD", 140},
  {"/RES", 141},
  {"Normal", 142},
  {"TxData", 143},
  {"selects the output signal if the internal FSK mode is used", 144},
  {"Modem Disc", 145},
  {"DTMF Keypad", 146},
  {"Keys transmitted", 147},
  {"Clear", 148},
  {"Keypad", 149},
  {"A", 150},
  {"B", 151},
  {"C", 152},
  {"*", 153},
  {"#", 154},
  {"D", 155},
  {"Baycom USB Speaker Scope", 156},
  {"Baycom USB Speaker Spectrum", 157},
  {"Baycom USB Microphone Scope", 158},
  {"Baycom USB Microphone Spectrum", 159},
  {"Baycom USB Adapter Parameters", 160},
  {"Enable PTT for Packet", 161},
  {"Baycom USB Audio Power Meters", 162},
  {"Mic", 163},
  {"Spkr", 164},
  {"N", 165},
  {"Couldn't find pixmap file: %s", 166},
  {"Error loading pixmap file: %s", 167},
};

int _msg_tbl_length = 167;
