EXPORTS
	AcquireSCMLock
	AddMiniIconToList
	AddTagToGroupOrderListEntry
	AppendStringToMultiSz
	AssertFail
	CaptureAndConvertAnsiArg
	CaptureStringArg
	CenterWindowRelativeToParent
	ConcatenatePaths
	DelayedMove
	DelimStringToMultiSz
	DestroyTextFileReadBuffer
	DoInstallActionWithParams
	DoesUserHavePrivilege
	DuplicateString
	EnablePrivilege
	ExtensionPropSheetPageProc
	FileExists
	FreeStringArray
	GetNewInfName
	GetSetFileTimestamp
	GetVersionInfoFromImage
	InfIsFromOemLocation
	InstallHinfSection
	InstallHinfSectionA
	InstallHinfSectionW
	InstallStop
	IsUserAdmin
	LookUpStringInTable
	MemoryInitialize
	MultiByteToUnicode
	MultiSzFromSearchControl
	MyFree
	MyGetFileTitle
	MyMalloc
	MyRealloc
	OpenAndMapFileForRead
	OutOfMemory
	QueryMultiSzValueToArray
	QueryRegistryValue
	ReadAsciiOrUnicodeTextFile
	RegistryDelnode
	RetreiveFileSecurity
	RetrieveServiceConfig
	SearchForInfFile
	SetArrayToMultiSzValue
	SetupAddInstallSectionToDiskSpaceListA
	SetupAddInstallSectionToDiskSpaceListW
	SetupAddSectionToDiskSpaceListA
	SetupAddSectionToDiskSpaceListW
	SetupAddToDiskSpaceListA
	SetupAddToDiskSpaceListW
	SetupAddToSourceListA
	SetupAddToSourceListW
	SetupAdjustDiskSpaceListA
	SetupAdjustDiskSpaceListW
	SetupCancelTemporarySourceList
	SetupCloseFileQueue
	SetupCloseInfFile
	SetupCloseLog
	SetupCommitFileQueue
	SetupCommitFileQueueA
	SetupCommitFileQueueW
	SetupCopyErrorA
	SetupCopyErrorW
	SetupCopyOEMInfA
	SetupCopyOEMInfW
	SetupCreateDiskSpaceListA
	SetupCreateDiskSpaceListW
	SetupDecompressOrCopyFileA
	SetupDecompressOrCopyFileW
	SetupDefaultQueueCallback
	SetupDefaultQueueCallbackA
	SetupDefaultQueueCallbackW
	SetupDeleteErrorA
	SetupDeleteErrorW
	SetupDestroyDiskSpaceList
	SetupDiAskForOEMDisk
	SetupDiBuildClassInfoList
	SetupDiBuildClassInfoListExA
	SetupDiBuildClassInfoListExW
	SetupDiBuildDriverInfoList
	SetupDiCallClassInstaller
	SetupDiCancelDriverInfoSearch
	SetupDiChangeState
	SetupDiClassGuidsFromNameA
	SetupDiClassGuidsFromNameExA
	SetupDiClassGuidsFromNameExW
	SetupDiClassGuidsFromNameW
	SetupDiClassNameFromGuidA
	SetupDiClassNameFromGuidExA
	SetupDiClassNameFromGuidExW
	SetupDiClassNameFromGuidW
	SetupDiCreateDevRegKeyA
	SetupDiCreateDevRegKeyW
	SetupDiCreateDeviceInfoA
	SetupDiCreateDeviceInfoList
	SetupDiCreateDeviceInfoListExA
	SetupDiCreateDeviceInfoListExW
	SetupDiCreateDeviceInfoW
	SetupDiCreateDeviceInterfaceA
	SetupDiCreateDeviceInterfaceRegKeyA
	SetupDiCreateDeviceInterfaceRegKeyW
	SetupDiCreateDeviceInterfaceW
	SetupDiDeleteDevRegKey
	SetupDiDeleteDeviceInfo
	SetupDiDeleteDeviceInterfaceData
	SetupDiDeleteDeviceInterfaceRegKey
	SetupDiDestroyClassImageList
	SetupDiDestroyDeviceInfoList
	SetupDiDestroyDriverInfoList
	SetupDiDrawMiniIcon
	SetupDiEnumDeviceInfo
	SetupDiEnumDeviceInterfaces
	SetupDiEnumDriverInfoA
	SetupDiEnumDriverInfoW
	SetupDiGetActualSectionToInstallA
	SetupDiGetActualSectionToInstallW
	SetupDiGetClassBitmapIndex
	SetupDiGetClassDescriptionA
	SetupDiGetClassDescriptionExA
	SetupDiGetClassDescriptionExW
	SetupDiGetClassDescriptionW
	SetupDiGetClassDevPropertySheetsA
	SetupDiGetClassDevPropertySheetsW
	SetupDiGetClassDevsA
	SetupDiGetClassDevsExA
	SetupDiGetClassDevsExW
	SetupDiGetClassDevsW
	SetupDiGetClassImageIndex
	SetupDiGetClassImageList
	SetupDiGetClassImageListExA
	SetupDiGetClassImageListExW
	SetupDiGetClassInstallParamsA
	SetupDiGetClassInstallParamsW
	SetupDiGetDeviceInfoListClass
	SetupDiGetDeviceInfoListDetailA
	SetupDiGetDeviceInfoListDetailW
	SetupDiGetDeviceInstallParamsA
	SetupDiGetDeviceInstallParamsW
	SetupDiGetDeviceInstanceIdA
	SetupDiGetDeviceInstanceIdW
	SetupDiGetDeviceInterfaceAlias
	SetupDiGetDeviceInterfaceDetailA
	SetupDiGetDeviceInterfaceDetailW
	SetupDiGetDeviceRegistryPropertyA@28
	SetupDiGetDeviceRegistryPropertyW
	SetupDiGetDriverInfoDetailA@24
	SetupDiGetDriverInfoDetailW
	SetupDiGetDriverInstallParamsA
	SetupDiGetDriverInstallParamsW
	SetupDiGetHwProfileFriendlyNameA
	SetupDiGetHwProfileFriendlyNameExA
	SetupDiGetHwProfileFriendlyNameExW
	SetupDiGetHwProfileFriendlyNameW
	SetupDiGetHwProfileList
	SetupDiGetHwProfileListExA
	SetupDiGetHwProfileListExW
	SetupDiGetINFClassA
	SetupDiGetINFClassW
	SetupDiGetSelectedDevice
	SetupDiGetSelectedDriverA@12
	SetupDiGetSelectedDriverW
	SetupDiGetWizardPage
	SetupDiInstallClassA
	SetupDiInstallClassExA
	SetupDiInstallClassExW
	SetupDiInstallClassW
	SetupDiInstallDevice
	SetupDiInstallDeviceInterfaces
	SetupDiInstallDriverFiles
	SetupDiLoadClassIcon
	SetupDiMoveDuplicateDevice
	SetupDiOpenClassRegKey
	SetupDiOpenClassRegKeyExA
	SetupDiOpenClassRegKeyExW
	SetupDiOpenDevRegKey
	SetupDiOpenDeviceInfoA
	SetupDiOpenDeviceInfoW
	SetupDiOpenDeviceInterfaceA
	SetupDiOpenDeviceInterfaceRegKey
	SetupDiOpenDeviceInterfaceW
	SetupDiRegisterCoDeviceInstallers
	SetupDiRegisterDeviceInfo
	SetupDiRemoveDevice
	SetupDiRemoveDeviceInterface
	SetupDiSelectBestCompatDrv
	SetupDiSelectDevice
	SetupDiSelectOEMDrv
	SetupDiSetClassInstallParamsA
	SetupDiSetClassInstallParamsW
	SetupDiSetDeviceInstallParamsA
	SetupDiSetDeviceInstallParamsW
	SetupDiSetDeviceRegistryPropertyA@20
	SetupDiSetDeviceRegistryPropertyW
	SetupDiSetDriverInstallParamsA
	SetupDiSetDriverInstallParamsW
	SetupDiSetSelectedDevice
	SetupDiSetSelectedDriverA
	SetupDiSetSelectedDriverW
	SetupDiUnremoveDevice
	SetupDuplicateDiskSpaceListA
	SetupDuplicateDiskSpaceListW
	SetupFindFirstLineA@16
	SetupFindFirstLineW
	SetupFindNextLine
	SetupFindNextMatchLineA
	SetupFindNextMatchLineW
	SetupFreeSourceListA
	SetupFreeSourceListW
	SetupGetBinaryField
	SetupGetFieldCount
	SetupGetFileCompressionInfoA
	SetupGetFileCompressionInfoW
	SetupGetInfFileListA
	SetupGetInfFileListW
	SetupGetInfInformationA
	SetupGetInfInformationW
	SetupGetInfSections
	SetupGetIntField
	SetupGetLineByIndexA
	SetupGetLineByIndexW
	SetupGetLineCountA
	SetupGetLineCountW
	SetupGetLineTextA
	SetupGetLineTextW
	SetupGetMultiSzFieldA
	SetupGetMultiSzFieldW
	SetupGetSourceFileLocationA
	SetupGetSourceFileLocationW
	SetupGetSourceFileSizeA
	SetupGetSourceFileSizeW
	SetupGetSourceInfoA
	SetupGetSourceInfoW
	SetupGetStringFieldA
	SetupGetStringFieldW
	SetupGetTargetPathA
	SetupGetTargetPathW
	SetupInitDefaultQueueCallback
	SetupInitDefaultQueueCallbackEx
	SetupInitializeFileLogA
	SetupInitializeFileLogW
	SetupInstallFileA
	SetupInstallFileExA
	SetupInstallFileExW
	SetupInstallFileW
	SetupInstallFilesFromInfSectionA
	SetupInstallFilesFromInfSectionW
	SetupInstallFromInfSectionA
	SetupInstallFromInfSectionW
	SetupInstallServicesFromInfSectionA
	SetupInstallServicesFromInfSectionExA
	SetupInstallServicesFromInfSectionExW
	SetupInstallServicesFromInfSectionW
	SetupIterateCabinetA
	SetupIterateCabinetW
	SetupLogErrorA
	SetupLogErrorW
	SetupLogFileA
	SetupLogFileW
	SetupOpenAppendInfFileA
	SetupOpenAppendInfFileW
	SetupOpenFileQueue
	SetupOpenInfFileA@16
	SetupOpenInfFileW
	SetupOpenLog
	SetupOpenMasterInf
	SetupPromptForDiskA
	SetupPromptForDiskW
	SetupPromptReboot
	SetupQueryDrivesInDiskSpaceListA
	SetupQueryDrivesInDiskSpaceListW
	SetupQueryFileLogA
	SetupQueryFileLogW
	SetupQueryInfFileInformationA
	SetupQueryInfFileInformationW
	SetupQueryInfVersionInformationA
	SetupQueryInfVersionInformationW
	SetupQuerySourceListA
	SetupQuerySourceListW
	SetupQuerySpaceRequiredOnDriveA
	SetupQuerySpaceRequiredOnDriveW
	SetupQueueCopyA
	SetupQueueCopySectionA
	SetupQueueCopySectionW
	SetupQueueCopyW
	SetupQueueDefaultCopyA
	SetupQueueDefaultCopyW
	SetupQueueDeleteA
	SetupQueueDeleteSectionA
	SetupQueueDeleteSectionW
	SetupQueueDeleteW
	SetupQueueRenameA
	SetupQueueRenameSectionA
	SetupQueueRenameSectionW
	SetupQueueRenameW
	SetupRemoveFileLogEntryA
	SetupRemoveFileLogEntryW
	SetupRemoveFromDiskSpaceListA
	SetupRemoveFromDiskSpaceListW
	SetupRemoveFromSourceListA
	SetupRemoveFromSourceListW
	SetupRemoveInstallSectionFromDiskSpaceListA
	SetupRemoveInstallSectionFromDiskSpaceListW
	SetupRemoveSectionFromDiskSpaceListA
	SetupRemoveSectionFromDiskSpaceListW
	SetupRenameErrorA
	SetupRenameErrorW
	SetupScanFileQueue
	SetupScanFileQueueA
	SetupScanFileQueueW
	SetupSetDirectoryIdA
	SetupSetDirectoryIdExA
	SetupSetDirectoryIdExW
	SetupSetDirectoryIdW
	SetupSetPlatformPathOverrideA
	SetupSetPlatformPathOverrideW
	SetupSetSourceListA
	SetupSetSourceListW
	SetupTermDefaultQueueCallback
	SetupTerminateFileLog
	ShouldDeviceBeExcluded
	StampFileSecurity
	StringTableAddString
	StringTableAddStringEx
	StringTableDestroy
	StringTableDuplicate
	StringTableEnum
	StringTableGetExtraData
	StringTableInitialize
	StringTableInitializeEx
	StringTableLookUpString
	StringTableLookUpStringEx
	StringTableSetExtraData
	StringTableStringFromId
	StringTableTrim
	TakeOwnershipOfFile
	UnicodeToMultiByte
	UnmapAndCloseFile
	pSetupDiGetDeviceInfoContext
	pSetupDiSetDeviceInfoContext
	pSetupDirectoryIdToPath
	pSetupGetField
	pSetupGetOsLoaderDriveAndPath
	pSetupGetVersionDatum
	pSetupGuidFromString
	pSetupIsGuidNull
	pSetupMakeSurePathExists
	pSetupSetSystemSourcePath
	pSetupStringFromGuid
