;LIBRARY FLEXNET_DRIVER
DESCRIPTION	"FlexNet Channel Driver"

VERSION 	1.0

CODE      READ SHARED EXECUTE
DATA      READ WRITE

EXPORTS

flexnet_driver

trxapi_init = win32shmem_init
trxapi_open_byindex = win32shmem_open_byindex
trxapi_close = win32shmem_close
trxapi_get_description = win32shmem_get_description
trxapi_get_state = win32shmem_get_state
trxapi_set_ptt = win32shmem_set_ptt
trxapi_set_frequency = win32shmem_set_frequency
trxapi_uart_send = win32shmem_uart_send
trxapi_uart_receive = win32shmem_uart_receive
trxapi_baycomusb_adapter_get_config = win32shmem_baycomusb_adapter_get_config
trxapi_baycomusb_adapter_set_config = win32shmem_baycomusb_adapter_set_config
trxapi_baycomusb_adapter_get_audiodevices = win32shmem_baycomusb_adapter_get_audiodevices
trxapi_channel_access_get_config = win32shmem_channel_access_get_config
trxapi_channel_access_set_config = win32shmem_channel_access_set_config
trxapi_interface_get_config = win32shmem_interface_get_config
trxapi_interface_set_config = win32shmem_interface_set_config
trxapi_modem_disconnect_get_config = win32shmem_modem_disconnect_get_config
trxapi_modem_disconnect_set_config = win32shmem_modem_disconnect_set_config
trxapi_get_audio_state = win32shmem_get_audio_state
trxapi_get_audio_mic_samples = win32shmem_get_audio_mic_samples
trxapi_get_audio_spk_samples = win32shmem_get_audio_spk_samples
trxapi_set_audio_dtmf = win32shmem_set_audio_dtmf

usb_show_device_descriptor
usb_setmountpoint
usb_getmountpoint
usb_close
usb_open_bynumber
usb_open
usb_control_msg
usb_bulk_msg
usb_resetep
usb_setconfiguration
usb_setinterface
usb_getdevicedescriptor
usb_claiminterface
usb_releaseinterface
usb_discsignal
usb_submiturb
usb_discardurb
usb_reapurb
usb_getfd

vsnprintf

server_entry_cmdline = server_entry_cmdline@16
server_entry_parse_argv
server_entry_run
server_entry_quit

