/*
 * This file was generated by orbit-idl - DO NOT EDIT!
 */

#include <glib.h>
#define ORBIT_IDL_SERIAL 9
#include <orb/orbit.h>

#ifndef trxctrl_H
#define trxctrl_H 1
#ifdef __cplusplus
extern "C"
{
#endif				/* __cplusplus */

/** typedefs **/
#if !defined(_Trx_frequency_defined)
#define _Trx_frequency_defined 1
   typedef CORBA_unsigned_long_long Trx_frequency;
#if !defined(TC_IMPL_TC_Trx_frequency_0)
#define TC_IMPL_TC_Trx_frequency_0 't'
#define TC_IMPL_TC_Trx_frequency_1 'r'
#define TC_IMPL_TC_Trx_frequency_2 'x'
#define TC_IMPL_TC_Trx_frequency_3 'c'
#define TC_IMPL_TC_Trx_frequency_4 't'
#define TC_IMPL_TC_Trx_frequency_5 'r'
#define TC_IMPL_TC_Trx_frequency_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_frequency_struct;
#define TC_Trx_frequency ((CORBA_TypeCode)&TC_Trx_frequency_struct)
#endif
#endif
#if !defined(_Trx_signalstrength_defined)
#define _Trx_signalstrength_defined 1
   typedef CORBA_float Trx_signalstrength;
#if !defined(TC_IMPL_TC_Trx_signalstrength_0)
#define TC_IMPL_TC_Trx_signalstrength_0 't'
#define TC_IMPL_TC_Trx_signalstrength_1 'r'
#define TC_IMPL_TC_Trx_signalstrength_2 'x'
#define TC_IMPL_TC_Trx_signalstrength_3 'c'
#define TC_IMPL_TC_Trx_signalstrength_4 't'
#define TC_IMPL_TC_Trx_signalstrength_5 'r'
#define TC_IMPL_TC_Trx_signalstrength_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_signalstrength_struct;
#define TC_Trx_signalstrength ((CORBA_TypeCode)&TC_Trx_signalstrength_struct)
#endif
#endif
#if !defined(_Trx_TrxBandDescription_defined)
#define _Trx_TrxBandDescription_defined 1
   typedef struct
   {
      Trx_frequency low;
      Trx_frequency high;
      Trx_frequency step;
   }
   Trx_TrxBandDescription;

#if !defined(TC_IMPL_TC_Trx_TrxBandDescription_0)
#define TC_IMPL_TC_Trx_TrxBandDescription_0 't'
#define TC_IMPL_TC_Trx_TrxBandDescription_1 'r'
#define TC_IMPL_TC_Trx_TrxBandDescription_2 'x'
#define TC_IMPL_TC_Trx_TrxBandDescription_3 'c'
#define TC_IMPL_TC_Trx_TrxBandDescription_4 't'
#define TC_IMPL_TC_Trx_TrxBandDescription_5 'r'
#define TC_IMPL_TC_Trx_TrxBandDescription_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxBandDescription_struct;
#define TC_Trx_TrxBandDescription ((CORBA_TypeCode)&TC_Trx_TrxBandDescription_struct)
#endif
   extern Trx_TrxBandDescription *Trx_TrxBandDescription__alloc(void);
   extern gpointer Trx_TrxBandDescription__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(ORBIT_DECL_CORBA_sequence_Trx_TrxBandDescription) && !defined(_CORBA_sequence_Trx_TrxBandDescription_defined)
#define ORBIT_DECL_CORBA_sequence_Trx_TrxBandDescription 1
#define _CORBA_sequence_Trx_TrxBandDescription_defined 1
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_0 't'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_1 'r'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_2 'x'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_3 'c'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_4 't'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_5 'r'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_6 'l'
   typedef struct
   {
      CORBA_unsigned_long _maximum,
       _length;
      Trx_TrxBandDescription *_buffer;
      CORBA_boolean _release;
   }
   CORBA_sequence_Trx_TrxBandDescription;
#if !defined(TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_0)
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_0 't'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_1 'r'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_2 'x'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_3 'c'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_4 't'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_5 'r'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_CORBA_sequence_Trx_TrxBandDescription_struct;
#define TC_CORBA_sequence_Trx_TrxBandDescription ((CORBA_TypeCode)&TC_CORBA_sequence_Trx_TrxBandDescription_struct)
#endif
   extern CORBA_sequence_Trx_TrxBandDescription
      *CORBA_sequence_Trx_TrxBandDescription__alloc(void);
   extern gpointer CORBA_sequence_Trx_TrxBandDescription__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
   Trx_TrxBandDescription
      *CORBA_sequence_Trx_TrxBandDescription_allocbuf(CORBA_unsigned_long
						      len);
#endif
#if !defined(ORBIT_DECL_CORBA_sequence_Trx_TrxBandDescription) && !defined(_CORBA_sequence_Trx_TrxBandDescription_defined)
#define ORBIT_DECL_CORBA_sequence_Trx_TrxBandDescription 1
#define _CORBA_sequence_Trx_TrxBandDescription_defined 1
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_0 't'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_1 'r'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_2 'x'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_3 'c'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_4 't'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_5 'r'
#define ORBIT_IMPL_CORBA_sequence_Trx_TrxBandDescription_6 'l'
   typedef struct
   {
      CORBA_unsigned_long _maximum,
       _length;
      Trx_TrxBandDescription *_buffer;
      CORBA_boolean _release;
   }
   CORBA_sequence_Trx_TrxBandDescription;
#if !defined(TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_0)
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_0 't'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_1 'r'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_2 'x'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_3 'c'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_4 't'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_5 'r'
#define TC_IMPL_TC_CORBA_sequence_Trx_TrxBandDescription_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_CORBA_sequence_Trx_TrxBandDescription_struct;
#define TC_CORBA_sequence_Trx_TrxBandDescription ((CORBA_TypeCode)&TC_CORBA_sequence_Trx_TrxBandDescription_struct)
#endif
   extern CORBA_sequence_Trx_TrxBandDescription
      *CORBA_sequence_Trx_TrxBandDescription__alloc(void);
   extern gpointer CORBA_sequence_Trx_TrxBandDescription__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
   Trx_TrxBandDescription
      *CORBA_sequence_Trx_TrxBandDescription_allocbuf(CORBA_unsigned_long
						      len);
#endif
#if !defined(_Trx_TrxDescription_defined)
#define _Trx_TrxDescription_defined 1
   typedef struct
   {
      CORBA_char *manufacturer;
      CORBA_char *model;
      CORBA_char *copyright;
      CORBA_char *instid;
      CORBA_sequence_Trx_TrxBandDescription rxbands;
      CORBA_sequence_Trx_TrxBandDescription txbands;
      Trx_signalstrength rssimin;
      Trx_signalstrength rssimax;
   }
   Trx_TrxDescription;

#if !defined(TC_IMPL_TC_Trx_TrxDescription_0)
#define TC_IMPL_TC_Trx_TrxDescription_0 't'
#define TC_IMPL_TC_Trx_TrxDescription_1 'r'
#define TC_IMPL_TC_Trx_TrxDescription_2 'x'
#define TC_IMPL_TC_Trx_TrxDescription_3 'c'
#define TC_IMPL_TC_Trx_TrxDescription_4 't'
#define TC_IMPL_TC_Trx_TrxDescription_5 'r'
#define TC_IMPL_TC_Trx_TrxDescription_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxDescription_struct;
#define TC_Trx_TrxDescription ((CORBA_TypeCode)&TC_Trx_TrxDescription_struct)
#endif
   extern Trx_TrxDescription *Trx_TrxDescription__alloc(void);
   extern gpointer Trx_TrxDescription__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_TrxMode_defined)
#define _Trx_TrxMode_defined 1
   typedef enum
   {
      Trx_trxunspec,
      Trx_trxpacket,
      Trx_trxaudio
   }
   Trx_TrxMode;
#if !defined(TC_IMPL_TC_Trx_TrxMode_0)
#define TC_IMPL_TC_Trx_TrxMode_0 't'
#define TC_IMPL_TC_Trx_TrxMode_1 'r'
#define TC_IMPL_TC_Trx_TrxMode_2 'x'
#define TC_IMPL_TC_Trx_TrxMode_3 'c'
#define TC_IMPL_TC_Trx_TrxMode_4 't'
#define TC_IMPL_TC_Trx_TrxMode_5 'r'
#define TC_IMPL_TC_Trx_TrxMode_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxMode_struct;
#define TC_Trx_TrxMode ((CORBA_TypeCode)&TC_Trx_TrxMode_struct)
#endif
#endif
#if !defined(_Trx_TrxState_defined)
#define _Trx_TrxState_defined 1
   typedef struct
   {
      Trx_TrxMode mode;
      Trx_frequency rxfreq;
      Trx_frequency txfreq;
      CORBA_boolean ptt;
      CORBA_boolean manualptt;
      CORBA_boolean dcd;
      Trx_signalstrength rssi;
   }
   Trx_TrxState;

#if !defined(TC_IMPL_TC_Trx_TrxState_0)
#define TC_IMPL_TC_Trx_TrxState_0 't'
#define TC_IMPL_TC_Trx_TrxState_1 'r'
#define TC_IMPL_TC_Trx_TrxState_2 'x'
#define TC_IMPL_TC_Trx_TrxState_3 'c'
#define TC_IMPL_TC_Trx_TrxState_4 't'
#define TC_IMPL_TC_Trx_TrxState_5 'r'
#define TC_IMPL_TC_Trx_TrxState_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxState_struct;
#define TC_Trx_TrxState ((CORBA_TypeCode)&TC_Trx_TrxState_struct)
#endif
   extern Trx_TrxState *Trx_TrxState__alloc(void);
   extern gpointer Trx_TrxState__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_BaycomUSBAdapterMode_defined)
#define _Trx_BaycomUSBAdapterMode_defined 1
   typedef enum
   {
      Trx_fsk,
      Trx_external,
      Trx_afsk,
      Trx_audio
   }
   Trx_BaycomUSBAdapterMode;
#if !defined(TC_IMPL_TC_Trx_BaycomUSBAdapterMode_0)
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_0 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_1 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_2 'x'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_3 'c'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_4 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_5 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterMode_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_BaycomUSBAdapterMode_struct;
#define TC_Trx_BaycomUSBAdapterMode ((CORBA_TypeCode)&TC_Trx_BaycomUSBAdapterMode_struct)
#endif
#endif
#if !defined(_Trx_BaycomUSBAdapterLoopback_defined)
#define _Trx_BaycomUSBAdapterLoopback_defined 1
   typedef enum
   {
      Trx_loopback_off,
      Trx_loopback_scrambler,
      Trx_loopback_modem
   }
   Trx_BaycomUSBAdapterLoopback;
#if !defined(TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_0)
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_0 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_1 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_2 'x'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_3 'c'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_4 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_5 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterLoopback_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_BaycomUSBAdapterLoopback_struct;
#define TC_Trx_BaycomUSBAdapterLoopback ((CORBA_TypeCode)&TC_Trx_BaycomUSBAdapterLoopback_struct)
#endif
#endif
#if !defined(_Trx_BaycomUSBAdapterAudioDev_defined)
#define _Trx_BaycomUSBAdapterAudioDev_defined 1
   typedef CORBA_char Trx_BaycomUSBAdapterAudioDev[64];
   typedef CORBA_char Trx_BaycomUSBAdapterAudioDev_slice;
#if !defined(TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_0)
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_0 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_1 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_2 'x'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_3 'c'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_4 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_5 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDev_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_BaycomUSBAdapterAudioDev_struct;
#define TC_Trx_BaycomUSBAdapterAudioDev ((CORBA_TypeCode)&TC_Trx_BaycomUSBAdapterAudioDev_struct)
#endif
   extern Trx_BaycomUSBAdapterAudioDev_slice
      *Trx_BaycomUSBAdapterAudioDev__alloc(void);
   extern gpointer Trx_BaycomUSBAdapterAudioDev__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_BaycomUSBAdapterConfig_defined)
#define _Trx_BaycomUSBAdapterConfig_defined 1
   typedef struct
   {
      Trx_BaycomUSBAdapterMode mode;
      CORBA_unsigned_long fclk;
      CORBA_unsigned_long bitraterx;
      CORBA_unsigned_long bitratetx;
      Trx_BaycomUSBAdapterLoopback loopback;
      CORBA_boolean pttmute;
      CORBA_unsigned_long filtmode;
      CORBA_unsigned_long samplerate;
      CORBA_float gain;
      Trx_BaycomUSBAdapterAudioDev audiodevin;
      Trx_BaycomUSBAdapterAudioDev audiodevout;
      Trx_signalstrength rfsquelch;
      CORBA_boolean audiosquelch;
   }
   Trx_BaycomUSBAdapterConfig;

#if !defined(TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_0)
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_0 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_1 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_2 'x'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_3 'c'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_4 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_5 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterConfig_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_BaycomUSBAdapterConfig_struct;
#define TC_Trx_BaycomUSBAdapterConfig ((CORBA_TypeCode)&TC_Trx_BaycomUSBAdapterConfig_struct)
#endif
   extern Trx_BaycomUSBAdapterConfig *Trx_BaycomUSBAdapterConfig__alloc(void);
   extern gpointer Trx_BaycomUSBAdapterConfig__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(ORBIT_DECL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev) && !defined(_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_defined)
#define ORBIT_DECL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev 1
#define _CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_defined 1
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 't'
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 'r'
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 'x'
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 'c'
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 't'
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 'r'
#define ORBIT_IMPL_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 'l'
   typedef struct
   {
      CORBA_unsigned_long _maximum,
       _length;
      Trx_BaycomUSBAdapterAudioDev *_buffer;
      CORBA_boolean _release;
   }
   CORBA_sequence_Trx_BaycomUSBAdapterAudioDev;
#if !defined(TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0)
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_0 't'
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_1 'r'
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_2 'x'
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_3 'c'
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_4 't'
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_5 'r'
#define TC_IMPL_TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_struct;
#define TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev ((CORBA_TypeCode)&TC_CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_struct)
#endif
   extern CORBA_sequence_Trx_BaycomUSBAdapterAudioDev
      *CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__alloc(void);
   extern gpointer CORBA_sequence_Trx_BaycomUSBAdapterAudioDev__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
   Trx_BaycomUSBAdapterAudioDev
      *CORBA_sequence_Trx_BaycomUSBAdapterAudioDev_allocbuf
      (CORBA_unsigned_long len);
#endif
#if !defined(_Trx_BaycomUSBAdapterAudioDevSequence_defined)
#define _Trx_BaycomUSBAdapterAudioDevSequence_defined 1
   typedef CORBA_sequence_Trx_BaycomUSBAdapterAudioDev
      Trx_BaycomUSBAdapterAudioDevSequence;
#if !defined(TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_0)
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_0 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_1 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_2 'x'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_3 'c'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_4 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_5 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevSequence_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_BaycomUSBAdapterAudioDevSequence_struct;
#define TC_Trx_BaycomUSBAdapterAudioDevSequence ((CORBA_TypeCode)&TC_Trx_BaycomUSBAdapterAudioDevSequence_struct)
#endif
   extern Trx_BaycomUSBAdapterAudioDevSequence
      *Trx_BaycomUSBAdapterAudioDevSequence__alloc(void);
   extern gpointer Trx_BaycomUSBAdapterAudioDevSequence__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_BaycomUSBAdapterAudioDevs_defined)
#define _Trx_BaycomUSBAdapterAudioDevs_defined 1
   typedef struct
   {
      Trx_BaycomUSBAdapterAudioDevSequence audiodevsin;
      Trx_BaycomUSBAdapterAudioDevSequence audiodevsout;
   }
   Trx_BaycomUSBAdapterAudioDevs;

#if !defined(TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_0)
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_0 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_1 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_2 'x'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_3 'c'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_4 't'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_5 'r'
#define TC_IMPL_TC_Trx_BaycomUSBAdapterAudioDevs_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_BaycomUSBAdapterAudioDevs_struct;
#define TC_Trx_BaycomUSBAdapterAudioDevs ((CORBA_TypeCode)&TC_Trx_BaycomUSBAdapterAudioDevs_struct)
#endif
   extern Trx_BaycomUSBAdapterAudioDevs
      *Trx_BaycomUSBAdapterAudioDevs__alloc(void);
   extern gpointer Trx_BaycomUSBAdapterAudioDevs__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_ChannelAccessConfig_defined)
#define _Trx_ChannelAccessConfig_defined 1
   typedef struct
   {
      CORBA_unsigned_long txdelay;
      CORBA_unsigned_long slottime;
      CORBA_unsigned_long ppersistence;
      CORBA_unsigned_long txtail;
      CORBA_boolean fullduplex;
   }
   Trx_ChannelAccessConfig;

#if !defined(TC_IMPL_TC_Trx_ChannelAccessConfig_0)
#define TC_IMPL_TC_Trx_ChannelAccessConfig_0 't'
#define TC_IMPL_TC_Trx_ChannelAccessConfig_1 'r'
#define TC_IMPL_TC_Trx_ChannelAccessConfig_2 'x'
#define TC_IMPL_TC_Trx_ChannelAccessConfig_3 'c'
#define TC_IMPL_TC_Trx_ChannelAccessConfig_4 't'
#define TC_IMPL_TC_Trx_ChannelAccessConfig_5 'r'
#define TC_IMPL_TC_Trx_ChannelAccessConfig_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_ChannelAccessConfig_struct;
#define TC_Trx_ChannelAccessConfig ((CORBA_TypeCode)&TC_Trx_ChannelAccessConfig_struct)
#endif
   extern Trx_ChannelAccessConfig *Trx_ChannelAccessConfig__alloc(void);
   extern gpointer Trx_ChannelAccessConfig__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_IpAddress_defined)
#define _Trx_IpAddress_defined 1
   typedef CORBA_octet Trx_IpAddress[4];
   typedef CORBA_octet Trx_IpAddress_slice;
#if !defined(TC_IMPL_TC_Trx_IpAddress_0)
#define TC_IMPL_TC_Trx_IpAddress_0 't'
#define TC_IMPL_TC_Trx_IpAddress_1 'r'
#define TC_IMPL_TC_Trx_IpAddress_2 'x'
#define TC_IMPL_TC_Trx_IpAddress_3 'c'
#define TC_IMPL_TC_Trx_IpAddress_4 't'
#define TC_IMPL_TC_Trx_IpAddress_5 'r'
#define TC_IMPL_TC_Trx_IpAddress_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_IpAddress_struct;
#define TC_Trx_IpAddress ((CORBA_TypeCode)&TC_Trx_IpAddress_struct)
#endif
   extern Trx_IpAddress_slice *Trx_IpAddress__alloc(void);
   extern gpointer Trx_IpAddress__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_InterfaceConfig_defined)
#define _Trx_InterfaceConfig_defined 1
   typedef struct
   {
      CORBA_char ifname[16];
      CORBA_char hwaddr[16];
      Trx_IpAddress ipaddr;
      Trx_IpAddress netmask;
      Trx_IpAddress broadcast;
      CORBA_char gwhwaddr[128];
      Trx_IpAddress gwipaddr;
      Trx_IpAddress gwnetmask;
      CORBA_char gwipmode;
   }
   Trx_InterfaceConfig;

#if !defined(TC_IMPL_TC_Trx_InterfaceConfig_0)
#define TC_IMPL_TC_Trx_InterfaceConfig_0 't'
#define TC_IMPL_TC_Trx_InterfaceConfig_1 'r'
#define TC_IMPL_TC_Trx_InterfaceConfig_2 'x'
#define TC_IMPL_TC_Trx_InterfaceConfig_3 'c'
#define TC_IMPL_TC_Trx_InterfaceConfig_4 't'
#define TC_IMPL_TC_Trx_InterfaceConfig_5 'r'
#define TC_IMPL_TC_Trx_InterfaceConfig_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_InterfaceConfig_struct;
#define TC_Trx_InterfaceConfig ((CORBA_TypeCode)&TC_Trx_InterfaceConfig_struct)
#endif
   extern Trx_InterfaceConfig *Trx_InterfaceConfig__alloc(void);
   extern gpointer Trx_InterfaceConfig__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_ModemDisconnectRxC_defined)
#define _Trx_ModemDisconnectRxC_defined 1
   typedef enum
   {
      Trx_mdisc_rxc_normal,
      Trx_mdisc_rxc_rxclock
   }
   Trx_ModemDisconnectRxC;
#if !defined(TC_IMPL_TC_Trx_ModemDisconnectRxC_0)
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_0 't'
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_1 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_2 'x'
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_3 'c'
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_4 't'
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_5 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectRxC_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectRxC_struct;
#define TC_Trx_ModemDisconnectRxC ((CORBA_TypeCode)&TC_Trx_ModemDisconnectRxC_struct)
#endif
#endif
#if !defined(_Trx_ModemDisconnectTxC_defined)
#define _Trx_ModemDisconnectTxC_defined 1
   typedef enum
   {
      Trx_mdisc_txc_normal,
      Trx_mdisc_txc_txclock
   }
   Trx_ModemDisconnectTxC;
#if !defined(TC_IMPL_TC_Trx_ModemDisconnectTxC_0)
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_0 't'
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_1 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_2 'x'
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_3 'c'
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_4 't'
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_5 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectTxC_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectTxC_struct;
#define TC_Trx_ModemDisconnectTxC ((CORBA_TypeCode)&TC_Trx_ModemDisconnectTxC_struct)
#endif
#endif
#if !defined(_Trx_ModemDisconnectTxD_defined)
#define _Trx_ModemDisconnectTxD_defined 1
   typedef enum
   {
      Trx_mdisc_txd_normal,
      Trx_mdisc_txd_txdata
   }
   Trx_ModemDisconnectTxD;
#if !defined(TC_IMPL_TC_Trx_ModemDisconnectTxD_0)
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_0 't'
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_1 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_2 'x'
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_3 'c'
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_4 't'
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_5 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectTxD_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_ModemDisconnectTxD_struct;
#define TC_Trx_ModemDisconnectTxD ((CORBA_TypeCode)&TC_Trx_ModemDisconnectTxD_struct)
#endif
#endif
#if !defined(_Trx_ModemDisconnectConfig_defined)
#define _Trx_ModemDisconnectConfig_defined 1
   typedef struct
   {
      CORBA_unsigned_long output;
      CORBA_unsigned_long direction;
      Trx_ModemDisconnectRxC rxc;
      Trx_ModemDisconnectTxC txc;
      Trx_ModemDisconnectTxD txd;
   }
   Trx_ModemDisconnectConfig;

#if !defined(TC_IMPL_TC_Trx_ModemDisconnectConfig_0)
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_0 't'
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_1 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_2 'x'
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_3 'c'
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_4 't'
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_5 'r'
#define TC_IMPL_TC_Trx_ModemDisconnectConfig_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_Trx_ModemDisconnectConfig_struct;
#define TC_Trx_ModemDisconnectConfig ((CORBA_TypeCode)&TC_Trx_ModemDisconnectConfig_struct)
#endif
   extern Trx_ModemDisconnectConfig *Trx_ModemDisconnectConfig__alloc(void);
   extern gpointer Trx_ModemDisconnectConfig__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_TrxAudioState_defined)
#define _Trx_TrxAudioState_defined 1
   typedef struct
   {
      CORBA_unsigned_long samplerate;
      CORBA_unsigned_long micbufsz;
      CORBA_unsigned_long micbufptr;
      CORBA_unsigned_long spkbufsz;
      CORBA_unsigned_long spkbufptr;
      CORBA_float micsigpwr;
      CORBA_float spksigpwr;
      CORBA_float spknoisepwr;
   }
   Trx_TrxAudioState;

#if !defined(TC_IMPL_TC_Trx_TrxAudioState_0)
#define TC_IMPL_TC_Trx_TrxAudioState_0 't'
#define TC_IMPL_TC_Trx_TrxAudioState_1 'r'
#define TC_IMPL_TC_Trx_TrxAudioState_2 'x'
#define TC_IMPL_TC_Trx_TrxAudioState_3 'c'
#define TC_IMPL_TC_Trx_TrxAudioState_4 't'
#define TC_IMPL_TC_Trx_TrxAudioState_5 'r'
#define TC_IMPL_TC_Trx_TrxAudioState_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxAudioState_struct;
#define TC_Trx_TrxAudioState ((CORBA_TypeCode)&TC_Trx_TrxAudioState_struct)
#endif
   extern Trx_TrxAudioState *Trx_TrxAudioState__alloc(void);
   extern gpointer Trx_TrxAudioState__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(ORBIT_DECL_CORBA_sequence_CORBA_octet) && !defined(_CORBA_sequence_CORBA_octet_defined)
#define ORBIT_DECL_CORBA_sequence_CORBA_octet 1
#define _CORBA_sequence_CORBA_octet_defined 1
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_0 't'
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_1 'r'
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_2 'x'
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_3 'c'
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_4 't'
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_5 'r'
#define ORBIT_IMPL_CORBA_sequence_CORBA_octet_6 'l'
   typedef struct
   {
      CORBA_unsigned_long _maximum,
       _length;
      CORBA_octet *_buffer;
      CORBA_boolean _release;
   }
   CORBA_sequence_CORBA_octet;
#if !defined(TC_IMPL_TC_CORBA_sequence_CORBA_octet_0)
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_0 't'
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_1 'r'
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_2 'x'
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_3 'c'
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_4 't'
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_5 'r'
#define TC_IMPL_TC_CORBA_sequence_CORBA_octet_6 'l'
   extern const struct CORBA_TypeCode_struct
      TC_CORBA_sequence_CORBA_octet_struct;
#define TC_CORBA_sequence_CORBA_octet ((CORBA_TypeCode)&TC_CORBA_sequence_CORBA_octet_struct)
#endif
   extern CORBA_sequence_CORBA_octet *CORBA_sequence_CORBA_octet__alloc(void);
   extern gpointer CORBA_sequence_CORBA_octet__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
   CORBA_octet *CORBA_sequence_CORBA_octet_allocbuf(CORBA_unsigned_long len);
#endif
#if !defined(_Trx_TrxAudioSamples_defined)
#define _Trx_TrxAudioSamples_defined 1
   typedef CORBA_sequence_CORBA_octet Trx_TrxAudioSamples;
#if !defined(TC_IMPL_TC_Trx_TrxAudioSamples_0)
#define TC_IMPL_TC_Trx_TrxAudioSamples_0 't'
#define TC_IMPL_TC_Trx_TrxAudioSamples_1 'r'
#define TC_IMPL_TC_Trx_TrxAudioSamples_2 'x'
#define TC_IMPL_TC_Trx_TrxAudioSamples_3 'c'
#define TC_IMPL_TC_Trx_TrxAudioSamples_4 't'
#define TC_IMPL_TC_Trx_TrxAudioSamples_5 'r'
#define TC_IMPL_TC_Trx_TrxAudioSamples_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxAudioSamples_struct;
#define TC_Trx_TrxAudioSamples ((CORBA_TypeCode)&TC_Trx_TrxAudioSamples_struct)
#endif
   extern Trx_TrxAudioSamples *Trx_TrxAudioSamples__alloc(void);
   extern gpointer Trx_TrxAudioSamples__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(_Trx_TrxAudioParams_defined)
#define _Trx_TrxAudioParams_defined 1
   typedef struct
   {
      Trx_signalstrength rfsquelch;
      CORBA_boolean audiosquelch;
   }
   Trx_TrxAudioParams;

#if !defined(TC_IMPL_TC_Trx_TrxAudioParams_0)
#define TC_IMPL_TC_Trx_TrxAudioParams_0 't'
#define TC_IMPL_TC_Trx_TrxAudioParams_1 'r'
#define TC_IMPL_TC_Trx_TrxAudioParams_2 'x'
#define TC_IMPL_TC_Trx_TrxAudioParams_3 'c'
#define TC_IMPL_TC_Trx_TrxAudioParams_4 't'
#define TC_IMPL_TC_Trx_TrxAudioParams_5 'r'
#define TC_IMPL_TC_Trx_TrxAudioParams_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxAudioParams_struct;
#define TC_Trx_TrxAudioParams ((CORBA_TypeCode)&TC_Trx_TrxAudioParams_struct)
#endif
   extern Trx_TrxAudioParams *Trx_TrxAudioParams__alloc(void);
   extern gpointer Trx_TrxAudioParams__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#define ex_Trx_TrxError "IDL:Trx/TrxError:1.0"
   void _ORBIT_Trx_TrxError_demarshal(GIOPRecvBuffer * _ORBIT_recv_buffer,
				      CORBA_Environment * ev);
   void _ORBIT_Trx_TrxError_marshal(GIOPSendBuffer * _ORBIT_send_buffer,
				    CORBA_Environment * ev);
#if !defined(_Trx_TrxError_defined)
#define _Trx_TrxError_defined 1
   typedef struct
   {
      CORBA_char *reason;
   }
   Trx_TrxError;

#if !defined(TC_IMPL_TC_Trx_TrxError_0)
#define TC_IMPL_TC_Trx_TrxError_0 't'
#define TC_IMPL_TC_Trx_TrxError_1 'r'
#define TC_IMPL_TC_Trx_TrxError_2 'x'
#define TC_IMPL_TC_Trx_TrxError_3 'c'
#define TC_IMPL_TC_Trx_TrxError_4 't'
#define TC_IMPL_TC_Trx_TrxError_5 'r'
#define TC_IMPL_TC_Trx_TrxError_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_TrxError_struct;
#define TC_Trx_TrxError ((CORBA_TypeCode)&TC_Trx_TrxError_struct)
#endif
   extern Trx_TrxError *Trx_TrxError__alloc(void);
   extern gpointer Trx_TrxError__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);	/* ORBit internal use */
#endif
#if !defined(ORBIT_DECL_Trx_Control) && !defined(_Trx_Control_defined)
#define ORBIT_DECL_Trx_Control 1
#define _Trx_Control_defined 1
#define Trx_Control__free CORBA_Object__free
   typedef CORBA_Object Trx_Control;
   extern CORBA_unsigned_long Trx_Control__classid;
#if !defined(TC_IMPL_TC_Trx_Control_0)
#define TC_IMPL_TC_Trx_Control_0 't'
#define TC_IMPL_TC_Trx_Control_1 'r'
#define TC_IMPL_TC_Trx_Control_2 'x'
#define TC_IMPL_TC_Trx_Control_3 'c'
#define TC_IMPL_TC_Trx_Control_4 't'
#define TC_IMPL_TC_Trx_Control_5 'r'
#define TC_IMPL_TC_Trx_Control_6 'l'
   extern const struct CORBA_TypeCode_struct TC_Trx_Control_struct;
#define TC_Trx_Control ((CORBA_TypeCode)&TC_Trx_Control_struct)
#endif
#endif

/** POA structures **/
   typedef struct
   {
      void *_private;
      void (*GetDescription) (PortableServer_Servant _servant,
			      Trx_TrxDescription ** desc,
			      CORBA_Environment * ev);
      void (*GetState) (PortableServer_Servant _servant, Trx_TrxState * state,
			CORBA_Environment * ev);
      void (*SetPTT) (PortableServer_Servant _servant,
		      const CORBA_boolean ptt, CORBA_Environment * ev);
      void (*SetFrequency) (PortableServer_Servant _servant,
			    const Trx_frequency rx, const Trx_frequency tx,
			    CORBA_Environment * ev);
      void (*UartSend) (PortableServer_Servant _servant,
			const CORBA_char * str, CORBA_Environment * ev);
      void (*UartReceive) (PortableServer_Servant _servant,
			   CORBA_unsigned_long * ptr, CORBA_char ** str,
			   CORBA_Environment * ev);
      void (*BaycomUSBAdapterGetConfig) (PortableServer_Servant _servant,
					 Trx_BaycomUSBAdapterConfig * cfg,
					 CORBA_Environment * ev);
      void (*BaycomUSBAdapterSetConfig) (PortableServer_Servant _servant,
					 const Trx_BaycomUSBAdapterConfig *
					 cfg, CORBA_Environment * ev);
      void (*BaycomUSBAdapterGetAudioDevices) (PortableServer_Servant
					       _servant,
					       Trx_BaycomUSBAdapterAudioDevs
					       ** devs,
					       CORBA_Environment * ev);
      void (*ChannelAccessGetConfig) (PortableServer_Servant _servant,
				      Trx_ChannelAccessConfig * cfg,
				      CORBA_Environment * ev);
      void (*ChannelAccessSetConfig) (PortableServer_Servant _servant,
				      const Trx_ChannelAccessConfig * cfg,
				      CORBA_Environment * ev);
      void (*InterfaceGetConfig) (PortableServer_Servant _servant,
				  Trx_InterfaceConfig * cfg,
				  CORBA_Environment * ev);
      void (*InterfaceSetConfig) (PortableServer_Servant _servant,
				  const Trx_InterfaceConfig * cfg,
				  CORBA_Environment * ev);
      void (*ModemDisconnectGetConfig) (PortableServer_Servant _servant,
					Trx_ModemDisconnectConfig * cfg,
					CORBA_Environment * ev);
      void (*ModemDisconnectSetConfig) (PortableServer_Servant _servant,
					const Trx_ModemDisconnectConfig * cfg,
					CORBA_Environment * ev);
      void (*GetAudioState) (PortableServer_Servant _servant,
			     Trx_TrxAudioState * state,
			     CORBA_Environment * ev);
      void (*GetAudioMicSamples) (PortableServer_Servant _servant,
				  const CORBA_unsigned_long ptr,
				  const CORBA_unsigned_long len,
				  Trx_TrxAudioSamples ** samples,
				  CORBA_Environment * ev);
      void (*GetAudioSpkSamples) (PortableServer_Servant _servant,
				  const CORBA_unsigned_long ptr,
				  const CORBA_unsigned_long len,
				  Trx_TrxAudioSamples ** samples,
				  CORBA_Environment * ev);
      void (*SetAudioDTMF) (PortableServer_Servant _servant,
			    const CORBA_long ch, CORBA_Environment * ev);
      void (*SetAudioParams) (PortableServer_Servant _servant,
			      const Trx_TrxAudioParams * params,
			      CORBA_Environment * ev);
   }
   POA_Trx_Control__epv;
   typedef struct
   {
      PortableServer_ServantBase__epv *_base_epv;
      POA_Trx_Control__epv *Trx_Control_epv;
   }
   POA_Trx_Control__vepv;
   typedef struct
   {
      void *_private;
      POA_Trx_Control__vepv *vepv;
   }
   POA_Trx_Control;
   extern void POA_Trx_Control__init(PortableServer_Servant servant,
				     CORBA_Environment * ev);
   extern void POA_Trx_Control__fini(PortableServer_Servant servant,
				     CORBA_Environment * ev);

/** prototypes **/
   void Trx_Control_GetDescription(Trx_Control _obj,
				   Trx_TrxDescription ** desc,
				   CORBA_Environment * ev);
   void Trx_Control_GetState(Trx_Control _obj, Trx_TrxState * state,
			     CORBA_Environment * ev);
   void Trx_Control_SetPTT(Trx_Control _obj, const CORBA_boolean ptt,
			   CORBA_Environment * ev);
   void Trx_Control_SetFrequency(Trx_Control _obj, const Trx_frequency rx,
				 const Trx_frequency tx,
				 CORBA_Environment * ev);
   void Trx_Control_UartSend(Trx_Control _obj, const CORBA_char * str,
			     CORBA_Environment * ev);
   void Trx_Control_UartReceive(Trx_Control _obj, CORBA_unsigned_long * ptr,
				CORBA_char ** str, CORBA_Environment * ev);
   void Trx_Control_BaycomUSBAdapterGetConfig(Trx_Control _obj,
					      Trx_BaycomUSBAdapterConfig *
					      cfg, CORBA_Environment * ev);
   void Trx_Control_BaycomUSBAdapterSetConfig(Trx_Control _obj,
					      const Trx_BaycomUSBAdapterConfig
					      * cfg, CORBA_Environment * ev);
   void Trx_Control_BaycomUSBAdapterGetAudioDevices(Trx_Control _obj,
						    Trx_BaycomUSBAdapterAudioDevs
						    ** devs,
						    CORBA_Environment * ev);
   void Trx_Control_ChannelAccessGetConfig(Trx_Control _obj,
					   Trx_ChannelAccessConfig * cfg,
					   CORBA_Environment * ev);
   void Trx_Control_ChannelAccessSetConfig(Trx_Control _obj,
					   const Trx_ChannelAccessConfig *
					   cfg, CORBA_Environment * ev);
   void Trx_Control_InterfaceGetConfig(Trx_Control _obj,
				       Trx_InterfaceConfig * cfg,
				       CORBA_Environment * ev);
   void Trx_Control_InterfaceSetConfig(Trx_Control _obj,
				       const Trx_InterfaceConfig * cfg,
				       CORBA_Environment * ev);
   void Trx_Control_ModemDisconnectGetConfig(Trx_Control _obj,
					     Trx_ModemDisconnectConfig * cfg,
					     CORBA_Environment * ev);
   void Trx_Control_ModemDisconnectSetConfig(Trx_Control _obj,
					     const Trx_ModemDisconnectConfig *
					     cfg, CORBA_Environment * ev);
   void Trx_Control_GetAudioState(Trx_Control _obj, Trx_TrxAudioState * state,
				  CORBA_Environment * ev);
   void Trx_Control_GetAudioMicSamples(Trx_Control _obj,
				       const CORBA_unsigned_long ptr,
				       const CORBA_unsigned_long len,
				       Trx_TrxAudioSamples ** samples,
				       CORBA_Environment * ev);
   void Trx_Control_GetAudioSpkSamples(Trx_Control _obj,
				       const CORBA_unsigned_long ptr,
				       const CORBA_unsigned_long len,
				       Trx_TrxAudioSamples ** samples,
				       CORBA_Environment * ev);
   void Trx_Control_SetAudioDTMF(Trx_Control _obj, const CORBA_long ch,
				 CORBA_Environment * ev);
   void Trx_Control_SetAudioParams(Trx_Control _obj,
				   const Trx_TrxAudioParams * params,
				   CORBA_Environment * ev);

   void _ORBIT_skel_Trx_Control_GetDescription(POA_Trx_Control *
					       _ORBIT_servant,
					       GIOPRecvBuffer *
					       _ORBIT_recv_buffer,
					       CORBA_Environment * ev,
					       void (*_impl_GetDescription)
					       (PortableServer_Servant
						_servant,
						Trx_TrxDescription ** desc,
						CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_GetState(POA_Trx_Control * _ORBIT_servant,
					 GIOPRecvBuffer * _ORBIT_recv_buffer,
					 CORBA_Environment * ev,
					 void (*_impl_GetState)
					 (PortableServer_Servant _servant,
					  Trx_TrxState * state,
					  CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_SetPTT(POA_Trx_Control * _ORBIT_servant,
				       GIOPRecvBuffer * _ORBIT_recv_buffer,
				       CORBA_Environment * ev,
				       void (*_impl_SetPTT)
				       (PortableServer_Servant _servant,
					const CORBA_boolean ptt,
					CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_SetFrequency(POA_Trx_Control * _ORBIT_servant,
					     GIOPRecvBuffer *
					     _ORBIT_recv_buffer,
					     CORBA_Environment * ev,
					     void (*_impl_SetFrequency)
					     (PortableServer_Servant _servant,
					      const Trx_frequency rx,
					      const Trx_frequency tx,
					      CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_UartSend(POA_Trx_Control * _ORBIT_servant,
					 GIOPRecvBuffer * _ORBIT_recv_buffer,
					 CORBA_Environment * ev,
					 void (*_impl_UartSend)
					 (PortableServer_Servant _servant,
					  const CORBA_char * str,
					  CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_UartReceive(POA_Trx_Control * _ORBIT_servant,
					    GIOPRecvBuffer *
					    _ORBIT_recv_buffer,
					    CORBA_Environment * ev,
					    void (*_impl_UartReceive)
					    (PortableServer_Servant _servant,
					     CORBA_unsigned_long * ptr,
					     CORBA_char ** str,
					     CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_BaycomUSBAdapterGetConfig(POA_Trx_Control *
							  _ORBIT_servant,
							  GIOPRecvBuffer *
							  _ORBIT_recv_buffer,
							  CORBA_Environment *
							  ev,
							  void
							  (*_impl_BaycomUSBAdapterGetConfig)
							  (PortableServer_Servant
							   _servant,
							   Trx_BaycomUSBAdapterConfig
							   * cfg,
							   CORBA_Environment *
							   ev));
   void _ORBIT_skel_Trx_Control_BaycomUSBAdapterSetConfig(POA_Trx_Control *
							  _ORBIT_servant,
							  GIOPRecvBuffer *
							  _ORBIT_recv_buffer,
							  CORBA_Environment *
							  ev,
							  void
							  (*_impl_BaycomUSBAdapterSetConfig)
							  (PortableServer_Servant
							   _servant,
							   const
							   Trx_BaycomUSBAdapterConfig
							   * cfg,
							   CORBA_Environment *
							   ev));
   void
      _ORBIT_skel_Trx_Control_BaycomUSBAdapterGetAudioDevices(POA_Trx_Control
							      *
							      _ORBIT_servant,
							      GIOPRecvBuffer *
							      _ORBIT_recv_buffer,
							      CORBA_Environment
							      * ev,
							      void
							      (*_impl_BaycomUSBAdapterGetAudioDevices)
							      (PortableServer_Servant
							       _servant,
							       Trx_BaycomUSBAdapterAudioDevs
							       ** devs,
							       CORBA_Environment
							       * ev));
   void _ORBIT_skel_Trx_Control_ChannelAccessGetConfig(POA_Trx_Control *
						       _ORBIT_servant,
						       GIOPRecvBuffer *
						       _ORBIT_recv_buffer,
						       CORBA_Environment * ev,
						       void
						       (*_impl_ChannelAccessGetConfig)
						       (PortableServer_Servant
							_servant,
							Trx_ChannelAccessConfig
							* cfg,
							CORBA_Environment *
							ev));
   void _ORBIT_skel_Trx_Control_ChannelAccessSetConfig(POA_Trx_Control *
						       _ORBIT_servant,
						       GIOPRecvBuffer *
						       _ORBIT_recv_buffer,
						       CORBA_Environment * ev,
						       void
						       (*_impl_ChannelAccessSetConfig)
						       (PortableServer_Servant
							_servant,
							const
							Trx_ChannelAccessConfig
							* cfg,
							CORBA_Environment *
							ev));
   void _ORBIT_skel_Trx_Control_InterfaceGetConfig(POA_Trx_Control *
						   _ORBIT_servant,
						   GIOPRecvBuffer *
						   _ORBIT_recv_buffer,
						   CORBA_Environment * ev,
						   void
						   (*_impl_InterfaceGetConfig)
						   (PortableServer_Servant
						    _servant,
						    Trx_InterfaceConfig * cfg,
						    CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_InterfaceSetConfig(POA_Trx_Control *
						   _ORBIT_servant,
						   GIOPRecvBuffer *
						   _ORBIT_recv_buffer,
						   CORBA_Environment * ev,
						   void
						   (*_impl_InterfaceSetConfig)
						   (PortableServer_Servant
						    _servant,
						    const Trx_InterfaceConfig
						    * cfg,
						    CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_ModemDisconnectGetConfig(POA_Trx_Control *
							 _ORBIT_servant,
							 GIOPRecvBuffer *
							 _ORBIT_recv_buffer,
							 CORBA_Environment *
							 ev,
							 void
							 (*_impl_ModemDisconnectGetConfig)
							 (PortableServer_Servant
							  _servant,
							  Trx_ModemDisconnectConfig
							  * cfg,
							  CORBA_Environment *
							  ev));
   void _ORBIT_skel_Trx_Control_ModemDisconnectSetConfig(POA_Trx_Control *
							 _ORBIT_servant,
							 GIOPRecvBuffer *
							 _ORBIT_recv_buffer,
							 CORBA_Environment *
							 ev,
							 void
							 (*_impl_ModemDisconnectSetConfig)
							 (PortableServer_Servant
							  _servant,
							  const
							  Trx_ModemDisconnectConfig
							  * cfg,
							  CORBA_Environment *
							  ev));
   void _ORBIT_skel_Trx_Control_GetAudioState(POA_Trx_Control *
					      _ORBIT_servant,
					      GIOPRecvBuffer *
					      _ORBIT_recv_buffer,
					      CORBA_Environment * ev,
					      void (*_impl_GetAudioState)
					      (PortableServer_Servant
					       _servant,
					       Trx_TrxAudioState * state,
					       CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_GetAudioMicSamples(POA_Trx_Control *
						   _ORBIT_servant,
						   GIOPRecvBuffer *
						   _ORBIT_recv_buffer,
						   CORBA_Environment * ev,
						   void
						   (*_impl_GetAudioMicSamples)
						   (PortableServer_Servant
						    _servant,
						    const CORBA_unsigned_long
						    ptr,
						    const CORBA_unsigned_long
						    len,
						    Trx_TrxAudioSamples **
						    samples,
						    CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_GetAudioSpkSamples(POA_Trx_Control *
						   _ORBIT_servant,
						   GIOPRecvBuffer *
						   _ORBIT_recv_buffer,
						   CORBA_Environment * ev,
						   void
						   (*_impl_GetAudioSpkSamples)
						   (PortableServer_Servant
						    _servant,
						    const CORBA_unsigned_long
						    ptr,
						    const CORBA_unsigned_long
						    len,
						    Trx_TrxAudioSamples **
						    samples,
						    CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_SetAudioDTMF(POA_Trx_Control * _ORBIT_servant,
					     GIOPRecvBuffer *
					     _ORBIT_recv_buffer,
					     CORBA_Environment * ev,
					     void (*_impl_SetAudioDTMF)
					     (PortableServer_Servant _servant,
					      const CORBA_long ch,
					      CORBA_Environment * ev));
   void _ORBIT_skel_Trx_Control_SetAudioParams(POA_Trx_Control *
					       _ORBIT_servant,
					       GIOPRecvBuffer *
					       _ORBIT_recv_buffer,
					       CORBA_Environment * ev,
					       void (*_impl_SetAudioParams)
					       (PortableServer_Servant
						_servant,
						const Trx_TrxAudioParams *
						params,
						CORBA_Environment * ev));
#ifdef __cplusplus
}
#endif				/* __cplusplus */

#endif
#undef ORBIT_IDL_SERIAL
