#include "resource.h"

#include "windows.h"

#define IDC_STATIC -1

IDD_DIALOG1 DIALOGEX 0, 0, 232, 176
STYLE DS_MODALFRAME | DS_3DLOOK | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,175,12,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,175,37,50,14
    RTEXT           "Serial Number",IDC_STATIC,40,15,35,8,0,WS_EX_RIGHT
    COMBOBOX        IDC_SERIAL,77,12,50,145,CBS_DROPDOWNLIST | 
                    CBS_DISABLENOSCROLL | WS_VSCROLL | WS_TABSTOP
    RTEXT           "Mode",IDC_STATIC,40,36,35,8
    COMBOBOX        IDC_MODE,77,34,50,145,CBS_DROPDOWNLIST | 
                    CBS_DISABLENOSCROLL | WS_VSCROLL | WS_TABSTOP
    RTEXT           "Bitrate RX",IDC_STATIC,40,58,35,8
    EDITTEXT        IDC_BITRATERX,77,56,40,14,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "bit/s",IDC_STATIC,120,58,20,8
    RTEXT           "Bitrate TX",IDC_STATIC,40,80,35,8
    EDITTEXT        IDC_BITRATETX,77,78,40,14,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "bit/s",IDC_STATIC,120,80,20,8
    RTEXT           "TxDelay",IDC_STATIC,40,102,35,8
    EDITTEXT        IDC_TXDELAY,77,100,40,14,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "ms",IDC_STATIC,120,102,10,8
    CONTROL         "Full Duplex",IDC_FULLDUPLEX,"Button",BS_3STATE |
                    WS_TABSTOP,77,122,51,10
    CONTROL         "PTT Mute",IDC_PTTMUTE,"Button",BS_3STATE |
                    WS_TABSTOP,77,144,51,10
    CONTROL         "Author: Thomas Sailer, HB9JNX/AE4WA",IDC_STATIC,"Static",
                    SS_LEFTNOWORDWRAP | WS_GROUP,3,166,130,8
END

LANGUAGE LANG_GERMAN, SUBLANG_GERMAN

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,0,1
 PRODUCTVERSION 1,0,0,1
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x0L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", "\0"
            VALUE "CompanyName", "Baycom Hard- und Software GmbH\0"
            VALUE "FileDescription", "baycomusb\0"
            VALUE "FileVersion", "1, 0, 0, 1\0"
            VALUE "InternalName", "Baycom USB FlexNet driver\0"
            VALUE "LegalCopyright", "Copyright © 1999-2000\0"
            VALUE "LegalTrademarks", "\0"
            VALUE "OriginalFilename", "baycomusb.dll\0"
            VALUE "PrivateBuild", "\0"
            VALUE "ProductName", "USBFLEX\0"
            VALUE "ProductVersion", "1.00a\0"
            VALUE "SpecialBuild", "\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
